/*
	This file is part of CGP-Library
	Copyright (c) Andrew James Turner 2014, 2015 (andrew.turner@york.ac.uk)

    CGP-Library is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CGP-Library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with CGP-Library. If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <float.h>


#include "cgp.h"
#include "utils.h"
#include "node-functions.h"
#include "sm-functions.h"




/*
	Prototypes of functions used internally to CGP-Library
*/

/* chromosome functions */
void setChromosomeActiveNodes(struct chromosome *chromo);
static void recursivelySetActiveNodes(struct chromosome *chromo, int nodeIndex);
static void sortChromosomeArray(struct chromosome **chromoArray, int numChromos);
static void getBestChromosome(struct chromosome **parents, struct chromosome **children, int numParents, int numChildren, struct chromosome *best);
static void saveChromosomeLatexRecursive(struct chromosome *chromo, int index, FILE *fp);


/* getting gene value functions  */
static double getRandomConnectionWeight(double weightRange);
static int getRandomNodeInput(int numChromoInputs, int numNodes, int nodePosition, double recurrentConnectionProbability, int *smNodes);
static int getRandomFunction(int numFunctions);
static int getRandomChromosomeOutput(int numInputs, int numNodes, int shortcutConnections, int *smNodes);

/* function set functions */
static int addPresetFunctionToFunctionSet(struct parameters *params, char const *functionName);
static void copyFunctionSet(struct functionSet *funcSetDest, struct functionSet *funcSetSrc);
static void printFunctionSet(struct parameters *params);

/* results functions */
struct results* initialiseResults(struct parameters *params, int numRuns);

/* mutation functions  */
static void probabilisticMutation(struct parameters *params, struct chromosome *chromo);
static void pointMutation(struct parameters *params, struct chromosome *chromo);
static void pointMutationANN(struct parameters *params, struct chromosome *chromo);
static void probabilisticMutationOnlyActive(struct parameters *params, struct chromosome *chromo);
static void singleMutation(struct parameters *params, struct chromosome *chromo);

/* selection scheme functions */
static void selectFittest(struct parameters *params, struct chromosome **parents, struct chromosome **candidateChromos, int numParents, int numCandidateChromos);

/* reproduction scheme functions */
static void mutateRandomParent(struct parameters *params, struct chromosome **parents, struct chromosome **children, int numParents, int numChildren);

/* fitness function */
static double supervisedLearning(struct parameters *params, struct chromosome *chromo, struct dataSet *data);


/*
	parameters function definitions
*/

/*
	Initialises a parameter struct with default values. These
	values can be individually changed via set functions.
*/
DLL_EXPORT struct parameters *initialiseParameters(const int numInputs, const int numNodes, const int numOutputs, const int arity) {

	struct parameters *params;

	/* allocate memory for parameters */
	params = (struct parameters*)malloc(sizeof(struct parameters));

	/* Set default values */
	params->mu = 1;
	params->lambda = 4;
	params->evolutionaryStrategy = '+';
	params->mutationRate = 0.05;
	params->recurrentConnectionProbability = 0.0;
    params->infoNodeWeight = 1.0;
	params->connectionWeightRange = 1;
	params->shortcutConnections = 1;

	params->targetFitness = 0;

	params->updateFrequency = 1;

    params->numSmActivations = 1;

	setNumInputs(params, numInputs);
	setNumNodes(params, numNodes);
	setNumOutputs(params, numOutputs);
	setArity(params, arity);

	params->mutationType = probabilisticMutation;
	strncpy(params->mutationTypeName, "probabilistic", MUTATIONTYPENAMELENGTH);

	params->funcSet = (struct functionSet*)malloc(sizeof(struct functionSet));
	params->funcSet->numFunctions = 0;

	params->fitnessFunction = supervisedLearning;
	strncpy(params->fitnessFunctionName, "supervisedLearning", FITNESSFUNCTIONNAMELENGTH);

	params->selectionScheme = selectFittest;
	strncpy(params->selectionSchemeName, "selectFittest", SELECTIONSCHEMENAMELENGTH);

	params->reproductionScheme = mutateRandomParent;
	strncpy(params->reproductionSchemeName, "mutateRandomParent", REPRODUCTIONSCHEMENAMELENGTH);

	params->numThreads = 1;


    params->numDevRuns = 10;
    params->numAverage = 4;

	/* Seed the random number generator */
	srand(time(NULL));

	return params;
}


/*
	Frees the memory associated with the given parameter structure
*/
DLL_EXPORT void freeParameters(struct parameters *params) {

	/* attempt to prevent user double freeing */
	if (params == NULL) {
		printf("Warning: double freeing of parameters prevented.\n");
		return;
	}

	free(params->funcSet);
	free(params);
}

/*
	prints the given parameters to the terminal
*/
DLL_EXPORT void printParameters(struct parameters *params) {

	if (params == NULL) {
		printf("Error: cannot print uninitialised parameters.\nTerminating CGP-Library.\n");
		exit(0);
	}

	printf("-----------------------------------------------------------\n");
	printf("                       Parameters                          \n");
	printf("-----------------------------------------------------------\n");
	printf("Evolutionary Strategy:\t\t\t(%d%c%d)-ES\n", params->mu, params->evolutionaryStrategy, params->lambda);
	printf("Inputs:\t\t\t\t\t\t\t%d\n", params->numInputs);
	printf("Nodes:\t\t\t\t\t\t\t%d\n", params->numNodes);
	printf("Outputs:\t\t\t\t\t\t%d\n", params->numOutputs);
	printf("Node Arity:\t\t\t\t\t\t%d\n", params->arity);
	printf("Connection weights range:\t\t+/- %f\n", params->connectionWeightRange);
	printf("Mutation Type:\t\t\t\t\t%s\n", params->mutationTypeName);
	printf("Mutation rate:\t\t\t\t\t%f\n", params->mutationRate);
	printf("Recurrent Connection Prob.:\t\t%f\n", params->recurrentConnectionProbability);
	printf("Shortcut Connections:\t\t\t%d\n", params->shortcutConnections);
	printf("Fitness Function:\t\t\t\t%s\n", params->fitnessFunctionName);
	printf("Target Fitness:\t\t\t\t\t%f\n", params->targetFitness);
	printf("Selection scheme:\t\t\t\t%s\n", params->selectionSchemeName);
	printf("Reproduction scheme:\t\t\t%s\n", params->reproductionSchemeName);
	printf("Update frequency:\t\t\t\t%d\n", params->updateFrequency);
	printf("Threads:\t\t\t\t\t\t%d\n", params->numThreads);
    printf("SM activations per execution: \t%d\n", params->numSmActivations);
    printf("Developmental steps: \t\t\t%d\n", params->numDevRuns);
    printf("Evaluations to average over: \t%d\n", params->numAverage);
	printFunctionSet(params);
	printf("-----------------------------------------------------------\n\n");
}


/*
	saves the given parameters to the terminal in printable format not loadable by the program
*/
DLL_EXPORT void saveParameters(struct parameters *params, char* filename) {

    if (params == NULL) {
        printf("Error: cannot print uninitialised parameters.\nTerminating CGP-Library.\n");
        exit(0);
    }

    FILE *f = fopen(filename, "w");

    fprintf(f, "-----------------------------------------------------------\n");
    fprintf(f, "                       Parameters                          \n");
    fprintf(f, "-----------------------------------------------------------\n");
    fprintf(f, "Evolutionary Strategy:\t\t\t(%d%c%d)-ES\n", params->mu, params->evolutionaryStrategy, params->lambda);
    fprintf(f, "Inputs:\t\t\t\t\t\t\t%d\n", params->numInputs);
    fprintf(f, "Nodes:\t\t\t\t\t\t\t%d\n", params->numNodes);
    fprintf(f, "Outputs:\t\t\t\t\t\t%d\n", params->numOutputs);
    fprintf(f, "Node Arity:\t\t\t\t\t\t%d\n", params->arity);
    fprintf(f, "Connection weights range:\t\t+/- %f\n", params->connectionWeightRange);
    fprintf(f, "Mutation Type:\t\t\t\t\t%s\n", params->mutationTypeName);
    fprintf(f, "Mutation rate:\t\t\t\t\t%f\n", params->mutationRate);
    fprintf(f, "Recurrent Connection Prob.:\t\t%f\n", params->recurrentConnectionProbability);
    fprintf(f, "Shortcut Connections:\t\t\t%d\n", params->shortcutConnections);
    fprintf(f, "Fitness Function:\t\t\t\t%s\n", params->fitnessFunctionName);
    fprintf(f, "Target Fitness:\t\t\t\t\t%f\n", params->targetFitness);
    fprintf(f, "Selection scheme:\t\t\t\t%s\n", params->selectionSchemeName);
    fprintf(f, "Reproduction scheme:\t\t\t%s\n", params->reproductionSchemeName);
    fprintf(f, "Update frequency:\t\t\t\t%d\n", params->updateFrequency);
    fprintf(f, "Threads:\t\t\t\t\t\t%d\n", params->numThreads);
    fprintf(f, "SM activations per execution: \t%d\n", params->numSmActivations);
    fprintf(f, "Developmental steps: \t\t\t%d\n", params->numDevRuns);
    fprintf(f, "Evaluations to average over: \t%d\n", params->numAverage);
    //print function set
    int i;

    fprintf(f,"Function Set:");

    for (i = 0; i < params->funcSet->numFunctions; i++) {
        fprintf(f," %s", params->funcSet->functionNames[i]);
    }

    fprintf(f," (%d)\n", params->funcSet->numFunctions);

    fprintf(f, "-----------------------------------------------------------\n\n");

    fclose(f);
}

/*
	Adds the give pre-defined functions to the given function set. The
	functions must be given in the char array. The function names must
	be comma separated and contain no spaces i.e. "and,or".
*/
DLL_EXPORT void addNodeFunction(struct parameters *params, char const *functionNames) {

	char *pch;
	char functionNamesAsArray[FUNCTIONNAMELENGTH * FUNCTIONSETSIZE];

	/* make a local copy of the function names*/
	strncpy(functionNamesAsArray, functionNames, FUNCTIONNAMELENGTH * FUNCTIONSETSIZE);

	/* get the first function name */
	pch = strtok(functionNamesAsArray, ", ");

	/* while the function names char array contains function names */
	while (pch != NULL) {

		/* add the named function to the function set */
		addPresetFunctionToFunctionSet(params, pch);

		/* get the next function name */
		pch = strtok(NULL, ", ");
	}

	/* if the function set is empty give warning */
	if (params->funcSet->numFunctions == 0) {
		printf("Warning: No Functions added to function set.\n");
	}
}


/*
	Adds given node function to given function set with given name.
	Disallows exceeding the function set size.
*/
DLL_EXPORT void addCustomNodeFunction(struct parameters *params, double (*function)(const int numInputs, const double *inputs, const double *weights), char const *functionName, int maxNumInputs, struct chromosome* (*smFunction)(struct chromosome *chromo_in, int index, int sm_args[NUM_ARGUMENTS], struct parameters *param), double (*infoFunction)(struct chromosome* chromo, int index, double info_weight), node_type type) {

	if (params->funcSet->numFunctions >= FUNCTIONSETSIZE) {
		printf("Warning: functions set has reached maximum capacity (%d). Function '%s' not added.\n", FUNCTIONSETSIZE, functionName);
		return;
	}

	/* set the function name as the given function name */
	strncpy(params->funcSet->functionNames[params->funcSet->numFunctions], functionName, FUNCTIONNAMELENGTH);

	/* set the number of function inputs as the given number of function inputs */
	params->funcSet->maxNumInputs[params->funcSet->numFunctions] = maxNumInputs;

	/* add the given function to the function set */
	params->funcSet->functions[params->funcSet->numFunctions] = function;

    /* set if function is self modifying */
    params->funcSet->smFunctions[params->funcSet->numFunctions] = smFunction;

    params->funcSet->infoFunctions[params->funcSet->numFunctions] = infoFunction;

    params->funcSet->type[params->funcSet->numFunctions] = type;

	params->funcSet->numFunctions++;
}


/*
	used as an interface to adding pre-set node functions.
	returns one if successful, zero otherwise.
*/
static int addPresetFunctionToFunctionSet(struct parameters *params, char const *functionName) {

	int successfullyAdded = 1;

	/* Symbolic functions */

	if (strncmp(functionName, "add", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _add, "add", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "sub", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _sub, "sub", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "mul", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _mul, "mul", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "div", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _divide, "div", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "abs", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _absolute, "abs", 1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "sqrt", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _squareRoot, "sqrt", 1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "sq", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _square, "sq", 1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "cube", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _cube, "cube", 1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "pow", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _power, "pow", 2, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "exp", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _exponential, "exp", 1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "sin", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _sine, "sin", 1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "cos", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _cosine, "cos", 1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "tan", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _tangent, "tan", 1, NULL, NULL, COMP_NODE);
	}


	/* Boolean logic gates */

	else if (strncmp(functionName, "and", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _and, "and", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "nand", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _nand, "nand", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "or", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _or, "or", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "nor", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _nor, "nor", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "xor", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _xor, "xor", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "xnor", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _xnor, "xnor", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "not", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _not, "not", 1, NULL, NULL, COMP_NODE);
	}

	/* Neuron functions */

	else if (strncmp(functionName, "sig", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _sigmoid, "sig", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "gauss", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _gaussian, "gauss", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "step", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _step, "step", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "softsign", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _softsign, "soft", -1, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "tanh", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _hyperbolicTangent, "tanh", -1, NULL, NULL, COMP_NODE);
	}

	/* other */

	else if (strncmp(functionName, "rand", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _randFloat, "rand", 0, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "1", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _constOne, "1", 0, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "0", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _constZero, "0", 0, NULL, NULL, COMP_NODE);
	}
	else if (strncmp(functionName, "pi", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _constPI, "pi", 0, NULL, NULL, COMP_NODE);
	}
    else if (strncmp(functionName, "200", FUNCTIONNAMELENGTH) == 0) {
//+d        addCustomNodeFunction(params, _constTwoHundred, "200", 0, NULL, NULL, COMP_NODE);
    }
	else if (strncmp(functionName, "wire", FUNCTIONNAMELENGTH) == 0) {
		addCustomNodeFunction(params, _wire, "wire", 1, NULL, NULL, COMP_NODE);
	}

    /* sm-functions */

    else if (strncmp(functionName, "sm-add", FUNCTIONNAMELENGTH) == 0){
        addCustomNodeFunction(params, sm_comp, "sm-add", 2, _ADD, NULL, SM_NODE);
    }
    else if (strncmp(functionName, "sm-del", FUNCTIONNAMELENGTH) == 0){
        addCustomNodeFunction(params, sm_comp, "sm-del", 2, _DEL, NULL, SM_NODE);
    }
    else if (strncmp(functionName, "sm-mov", FUNCTIONNAMELENGTH) == 0){
        addCustomNodeFunction(params, sm_comp, "sm-mov", 2, _MOV, NULL, SM_NODE);
    }
    else if (strncmp(functionName, "sm-movr", FUNCTIONNAMELENGTH) == 0){
        addCustomNodeFunction(params, sm_comp, "sm-movr", 2, _MOVR, NULL, SM_NODE);
    }
    else if (strncmp(functionName, "sm-dup", FUNCTIONNAMELENGTH) == 0){
        addCustomNodeFunction(params, sm_comp, "sm-dup", 2, _DUP, NULL, SM_NODE);
    }
    else if (strncmp(functionName, "sm-dupr", FUNCTIONNAMELENGTH) == 0){
        addCustomNodeFunction(params, sm_comp, "sm-dupr", 2, _DUPR, NULL, SM_NODE);
    }

    /* info-functions */
    else if(strncmp(functionName, "index", FUNCTIONNAMELENGTH) == 0){
        addCustomNodeFunction(params, NULL, "index", 0, NULL, _INDEX, INFO_NODE);
    }
    else if(strncmp(functionName, "inputs", FUNCTIONNAMELENGTH) == 0){
        addCustomNodeFunction(params, NULL, "inputs", 0, NULL, _INPUTS, INFO_NODE);
    }
    else if(strncmp(functionName, "size", FUNCTIONNAMELENGTH) == 0){
        addCustomNodeFunction(params, NULL, "size", 0, NULL, _SIZE, INFO_NODE);
    }

	else {
		printf("Warning: function '%s' is not known and was not added.\n", functionName);
		successfullyAdded = 0;
	}

	return successfullyAdded;
}


/*
	clears the given function set of functions
*/
DLL_EXPORT void clearFunctionSet(struct parameters *params) {
	params->funcSet->numFunctions = 0;
}


/*
	sets num chromosome inputs in parameters
*/
DLL_EXPORT void setNumInputs(struct parameters *params, int numInputs) {

	/* error checking */
	if (numInputs <= 0) {
		printf("Error: number of chromosome inputs cannot be less than one; %d is invalid.\nTerminating CGP-Library.\n", numInputs);
		exit(0);
	}

	params->numInputs = numInputs;
}


/*
	sets num chromosome nodes in parameters
*/
DLL_EXPORT void setNumNodes(struct parameters *params, int numNodes) {

	/* error checking */
	if (numNodes < 0) {
		printf("Warning: number of chromosome nodes cannot be negative; %d is invalid.\nTerminating CGP-Library.\n", numNodes);
		exit(0);
	}

	params->numNodes = numNodes;
}


/*
	sets num chromosome outputs in parameters
*/
DLL_EXPORT void setNumOutputs(struct parameters *params, int numOutputs) {

	/* error checking */
	if (numOutputs < 0) {
		printf("Warning: number of chromosome outputs cannot be less than one; %d is invalid.\nTerminating CGP-Library.\n", numOutputs);
		exit(0);
	}

	params->numOutputs = numOutputs;
}


/*
	sets chromosome arity in parameters
*/
DLL_EXPORT void setArity(struct parameters *params, int arity) {

	/* error checking */
	if (arity < 0) {
		printf("Warning: node arity cannot be less than one; %d is invalid.\nTerminating CGP-Library.\n", arity);
		exit(0);
	}

	params->arity = arity;
}


/*
	Sets the mu value in given parameters to the new given value. If mu value
	is invalid a warning is displayed and the mu value is left unchanged.
*/
DLL_EXPORT void setMu(struct parameters *params, int mu) {

	if (mu > 0) {
		params->mu = mu;
	}
	else {
		printf("\nWarning: mu value '%d' is invalid. Mu value must have a value of one or greater. Mu value left unchanged as '%d'.\n", mu, params->mu);
	}
}


/*
	Sets the lambda value in given parameters to the new given value.
	If lambda value is invalid a warning is displayed and the lambda value
	is left unchanged.
*/
DLL_EXPORT void setLambda(struct parameters *params, int lambda) {

	if (lambda > 0) {
		params->lambda = lambda;
	}
	else {
		printf("\nWarning: lambda value '%d' is invalid. Lambda value must have a value of one or greater. Lambda value left unchanged as '%d'.\n", lambda, params->lambda);
	}
}


/*
	Sets the evolutionary strategy given in parameters to '+' or ','.
	If an invalid option is given a warning is displayed and the evolutionary
	strategy is left unchanged.
*/
DLL_EXPORT void setEvolutionaryStrategy(struct parameters *params, char evolutionaryStrategy) {

	if (evolutionaryStrategy == '+' || evolutionaryStrategy == ',') {
		params->evolutionaryStrategy = evolutionaryStrategy;
	}
	else {
		printf("\nWarning: the evolutionary strategy '%c' is invalid. The evolutionary strategy must be '+' or ','. The evolutionary strategy has been left unchanged as '%c'.\n", evolutionaryStrategy, params->evolutionaryStrategy);
	}
}


/*
	Sets the mutation rate given in parameters. If an invalid mutation
	rate is given a warning is displayed and the mutation rate is left
	unchanged.
*/
DLL_EXPORT void setMutationRate(struct parameters *params, double mutationRate) {

	if (mutationRate >= 0 && mutationRate <= 1) {
		params->mutationRate = mutationRate;
	}
	else {
		printf("\nWarning: mutation rate '%f' is invalid. The mutation rate must be in the range [0,1]. The mutation rate has been left unchanged as '%f'.\n", mutationRate, params->mutationRate);
	}
}


/*
	Sets the recurrent connection probability given in parameters. If an invalid
	value is given a warning is displayed and the value is left	unchanged.
*/
DLL_EXPORT void setRecurrentConnectionProbability(struct parameters *params, double recurrentConnectionProbability) {

	if (recurrentConnectionProbability >= 0 && recurrentConnectionProbability <= 1) {
		params->recurrentConnectionProbability = recurrentConnectionProbability;
	}
	else {
		printf("\nWarning: recurrent connection probability '%f' is invalid. The recurrent connection probability must be in the range [0,1]. The recurrent connection probability has been left unchanged as '%f'.\n", recurrentConnectionProbability, params->recurrentConnectionProbability);
	}
}


/*
	Sets the whether shortcut connections are used. If an invalid
	value is given a warning is displayed and the value is left	unchanged.
*/
DLL_EXPORT void setShortcutConnections(struct parameters *params, int shortcutConnections) {

	if (shortcutConnections == 0 || shortcutConnections == 1) {
		params->shortcutConnections = shortcutConnections;
	}
	else {
		printf("\nWarning: shortcut connection '%d' is invalid. The shortcut connections takes values 0 or 1. The shortcut connection has been left unchanged as '%d'.\n", shortcutConnections, params->shortcutConnections);
	}
}
/*
	Sets the output weight of info nodes in parameters.
*/
DLL_EXPORT void setNodeInfoWeight(struct parameters *params, double info_weight) {

    params->infoNodeWeight = info_weight;
}



/*
	Sets the connection weight range given in parameters.
*/
DLL_EXPORT void setConnectionWeightRange(struct parameters *params, double weightRange) {

	params->connectionWeightRange = weightRange;
}


/*
	sets the fitness function to the fitnessFunction passed. If the fitnessFunction is NULL
	then the default supervisedLearning fitness function is used.
*/
DLL_EXPORT void setCustomFitnessFunction(struct parameters *params, double (*fitnessFunction)(struct parameters *params, struct chromosome *chromo, struct dataSet *data), char const *fitnessFunctionName) {

	if (fitnessFunction == NULL) {
		params->fitnessFunction = supervisedLearning;
		strncpy(params->fitnessFunctionName, "supervisedLearning", FITNESSFUNCTIONNAMELENGTH);
	}
	else {
		params->fitnessFunction = fitnessFunction;
		strncpy(params->fitnessFunctionName, fitnessFunctionName, FITNESSFUNCTIONNAMELENGTH);
	}
}



/*
	sets the selection scheme used to select the parents from the candidate chromosomes. If the selectionScheme is NULL
	then the default selectFittest selection scheme is used.
*/
DLL_EXPORT void setCustomSelectionScheme(struct parameters *params, void (*selectionScheme)(struct parameters *params, struct chromosome **parents, struct chromosome **candidateChromos, int numParents, int numCandidateChromos), char const *selectionSchemeName) {

	if (selectionScheme == NULL) {
		params->selectionScheme = selectFittest;
		strncpy(params->selectionSchemeName, "selectFittest", SELECTIONSCHEMENAMELENGTH);
	}
	else {
		params->selectionScheme = selectionScheme;
		strncpy(params->selectionSchemeName, selectionSchemeName, SELECTIONSCHEMENAMELENGTH);
	}
}


/*
	sets the reproduction scheme used to select the parents from the candidate chromosomes. If the reproductionScheme is NULL
	then the default mutateRandomParent selection scheme is used.
*/

DLL_EXPORT void setCustomReproductionScheme(struct parameters *params, void (*reproductionScheme)(struct parameters *params, struct chromosome **parents, struct chromosome **children, int numParents, int numChildren), char const *reproductionSchemeName) {

	if (reproductionScheme == NULL) {
		params->reproductionScheme = mutateRandomParent;
		strncpy(params->reproductionSchemeName, "mutateRandomParent", REPRODUCTIONSCHEMENAMELENGTH);
	}
	else {
		params->reproductionScheme = reproductionScheme;
		strncpy(params->reproductionSchemeName, reproductionSchemeName, REPRODUCTIONSCHEMENAMELENGTH);
	}
}


/*
	Sets the target fitness
*/
DLL_EXPORT void setTargetFitness(struct parameters *params, double targetFitness) {

	params->targetFitness = targetFitness;
}


/*
	sets the mutation type in params
*/
DLL_EXPORT void setMutationType(struct parameters *params, char const *mutationType) {

	if (strncmp(mutationType, "probabilistic", MUTATIONTYPENAMELENGTH) == 0) {

		params->mutationType = probabilisticMutation;
		strncpy(params->mutationTypeName, "probabilistic", MUTATIONTYPENAMELENGTH);
	}

	else if (strncmp(mutationType, "point", MUTATIONTYPENAMELENGTH) == 0) {

		params->mutationType = pointMutation;
		strncpy(params->mutationTypeName, "point", MUTATIONTYPENAMELENGTH);
	}

	else if (strncmp(mutationType, "pointANN", MUTATIONTYPENAMELENGTH) == 0) {

		params->mutationType = pointMutationANN;
		strncpy(params->mutationTypeName, "pointANN", MUTATIONTYPENAMELENGTH);
	}

	else if (strncmp(mutationType, "onlyActive", MUTATIONTYPENAMELENGTH) == 0) {

		params->mutationType = probabilisticMutationOnlyActive;
		strncpy(params->mutationTypeName, "onlyActive", MUTATIONTYPENAMELENGTH);
	}

	else if (strncmp(mutationType, "single", MUTATIONTYPENAMELENGTH) == 0) {

		params->mutationType = singleMutation;
		strncpy(params->mutationTypeName, "single", MUTATIONTYPENAMELENGTH);
	}

	else {
		printf("\nWarning: mutation type '%s' is invalid. The mutation type must be 'probabilistic' or 'point'. The mutation type has been left unchanged as '%s'.\n", mutationType, params->mutationTypeName);
	}
}


/*
	Sets the update frequency in generations
*/
DLL_EXPORT void setUpdateFrequency(struct parameters *params, int updateFrequency) {

	if (updateFrequency < 0) {
		printf("Warning: update frequency of %d is invalid. Update frequency must be >= 0. Update frequency is left unchanged as %d.\n", updateFrequency, params->updateFrequency);
	}
	else {
		params->updateFrequency = updateFrequency;
	}
}




/*
	sets num chromosome inputs in parameters
*/
DLL_EXPORT void setNumThreads(struct parameters *params, int numThreads) {

	/* error checking */
	if (numThreads <= 0) {
		printf("Error: number threads cannot be less than one; %d is invalid. The number threads is left unchanged as %d.\n", numThreads, numThreads);
	}

	params->numThreads = numThreads;
}

/*
	sets num development runs in evaluation of chromosome
*/
DLL_EXPORT void setNumDevRuns(struct parameters *params, int devRuns) {

    /* error checking */
    if (devRuns <= 0) {
        printf("Error: number of dev runs cannot be less than one; %d is invalid. The number devRuns is left unchanged as %d.\n", devRuns, params->numDevRuns);
        return;
    }

    params->numDevRuns = devRuns;
}

/*
	sets num evaluation to average result over in evaluation of chromosome
*/
DLL_EXPORT void setNumAverage(struct parameters *params, int numAverage) {

    /* error checking */
    if (numAverage <= 0) {
        printf("Error: number of dev runs cannot be less than one; %d is invalid. The number devRuns is left unchanged as %d.\n", numAverage, params->numAverage);
        return;
    }

    params->numAverage = numAverage;
}



/*
	chromosome function definitions
*/


/*
	Returns a pointer to an initialised chromosome with values obeying the given parameters.
*/
DLL_EXPORT struct chromosome *initialiseChromosome(struct parameters *params) {

	struct chromosome *chromo;
	int i;

	/* check that funcSet contains functions */
	if (params->funcSet->numFunctions < 1) {
		printf("Error: chromosome not initialised due to empty functionSet.\nTerminating CGP-Library.\n");
		exit(0);
	}

	/* allocate memory for chromosome */
	chromo = (struct chromosome*)malloc(sizeof(struct chromosome));

	/* allocate memory for nodes */
	chromo->nodes = (struct node**)malloc(params->numNodes * sizeof(struct node*));

	/* allocate memory for outputNodes matrix */
	chromo->outputNodes = (int*)malloc(params->numOutputs * sizeof(int));

	/* allocate memory for active nodes matrix */
	chromo->activeNodes = (int*)malloc(params->numNodes * sizeof(int));

    /* allocate memory for sm nodes matrix */
    chromo->smNodes = (int*)calloc(params->numNodes, sizeof(int));

	/* allocate memory for chromosome outputValues */
	chromo->outputValues = (double*)malloc(params->numOutputs * sizeof(double));

	/* Initialise each of the chromosomes nodes */
	for (i = 0; i < params->numNodes; i++) {
		chromo->nodes[i] = initialiseNode(params->numInputs, params->numNodes, params->arity, params->funcSet->numFunctions, params->connectionWeightRange, params->recurrentConnectionProbability, i, chromo->smNodes);
        chromo->smNodes[i] = params->funcSet->type[chromo->nodes[i]->function] == SM_NODE ? 1 : 0;
	}


	/* set each of the chromosomes outputs */
	for (i = 0; i < params->numOutputs; i++) {
		chromo->outputNodes[i] = getRandomChromosomeOutput(params->numInputs, params->numNodes, params->shortcutConnections, chromo->smNodes);
	}

	/* set the number of inputs, nodes and outputs */
	chromo->numInputs = params->numInputs;
	chromo->numNodes = params->numNodes;
	chromo->numOutputs = params->numOutputs;
	chromo->arity = params->arity;
    chromo->numToDos = 0;
	chromo->recurrentConnectionProbability = params->recurrentConnectionProbability;

	/* set the number of active node to the number of nodes (all active) */
	chromo->numActiveNodes = params->numNodes;

	/* set the fitness to initial value */
	chromo->fitness = -1;

	/* copy the function set from the parameters to the chromosome */
	chromo->funcSet = (struct functionSet*)malloc(sizeof(struct functionSet));
	copyFunctionSet(chromo->funcSet, params->funcSet);

	/* set the active nodes in the newly generated chromosome */
	setChromosomeActiveNodes(chromo);

    checkChromosomeInputs(chromo);

	/* used interally when exicuting chromosome */
	chromo->nodeInputsHold = (double*)malloc(params->arity * sizeof(double));

	return chromo;
}


/*
	Reads in saved chromosomes
*/
DLL_EXPORT struct chromosome* initialiseChromosomeFromFile(char const *file) {

	int i, j;

	FILE *fp;
	struct chromosome *chromo;
	struct parameters *params;

	char *line, *record;
	char funcName[FUNCTIONNAMELENGTH];
	char buffer[1024];

	int numInputs, numNodes, numOutputs, arity;

	/* open the chromosome file */
	fp = fopen(file, "r");

	/* ensure that the file was opened correctly */
	if (fp == NULL) {
		printf("Warning: cannot open chromosome: '%s'. Chromosome was not open.\n", file);
		return NULL;
	}

	/* get num inputs */
	line = fgets(buffer, sizeof(buffer), fp);
	if (line == NULL) {/*error*/}
	record = strtok(line, ",");
	record = strtok(NULL, ",");
	numInputs = atoi(record);

	/* get num nodes */
	line = fgets(buffer, sizeof(buffer), fp);
	if (line == NULL) {/*error*/}
	record = strtok(line, ",");
	record = strtok(NULL, ",");
	numNodes = atoi(record);

	/* get num outputs */
	line = fgets(buffer, sizeof(buffer), fp);
	if (line == NULL) {/*error*/}
	record = strtok(line, ",");
	record = strtok(NULL, ",");
	numOutputs = atoi(record);

	/* get arity */
	line = fgets(buffer, sizeof(buffer), fp);
	if (line == NULL) {/*error*/}
	record = strtok(line, ",");
	record = strtok(NULL, ",");
	arity = atoi(record);

	/* initialise parameters  */
	params = initialiseParameters(numInputs, numNodes, numOutputs, arity);

	/* get and set node functions */
	line = fgets(buffer, sizeof(buffer), fp);
	if (line == NULL) {/*error*/}
	record = strtok(line, ",\n");
	record = strtok(NULL, ",\n");

	/* for each function name */
	while ( record != NULL) {

		strncpy(funcName, record, FUNCTIONNAMELENGTH);

		/* can only load functions defined within CGP-Library */
		if (addPresetFunctionToFunctionSet(params, funcName) == 0) {
			printf("Error: cannot load chromosome which contains custom node functions.\n");
			printf("Terminating CGP-Library.\n");
			freeParameters(params);
			exit(0);
		}

		record = strtok(NULL, ",\n");
	}

	/* initialise a chromosome beased on the parameters accociated with given chromosome */
	chromo = initialiseChromosome(params);

	/* set the node parameters */
	for (i = 0; i < numNodes; i++) {

		/* get the function gene */
		line = fgets(buffer, sizeof(buffer), fp);
		record = strtok(line, ",\n");
		chromo->nodes[i]->function = atoi(record);

		for (j = 0; j < arity; j++) {
			line = fgets(buffer, sizeof(buffer), fp);
			sscanf(line, "%d,%lf", &chromo->nodes[i]->inputs[j], &chromo->nodes[i]->weights[j]);
		}

        for (j = 0; j < NUM_ARGUMENTS; j++){
            line = fgets(buffer, sizeof(buffer), fp);
            sscanf(line, "%d", &chromo->nodes[i]->arguments[j]);
        }

	}

	/* set the outputs */
	line = fgets(buffer, sizeof(buffer), fp);
	record = strtok(line, ",\n");
	chromo->outputNodes[0] = atoi(record);

	for (i = 1; i < numOutputs; i++) {
		record = strtok(NULL, ",\n");
		chromo->outputNodes[i] = atoi(record);
	}

	fclose(fp);
	freeParameters(params);

	/* set the active nodes in the copied chromosome */
	setChromosomeActiveNodes(chromo);

	return chromo;
}


/*
	Returns a pointer to an initialised chromosome with values obeying the given parameters.
*/
DLL_EXPORT struct chromosome *initialiseChromosomeFromChromosome(struct chromosome *chromo) {

	struct chromosome *chromoNew;
	int i;

	/* check that funcSet contains functions*/
	if (chromo == NULL) {
		printf("Error: cannot initialise chromosome from uninitialised chromosome.\nTerminating CGP-Library.\n");
		exit(0);
	}

	/* allocate memory for chromosome */
	chromoNew = (struct chromosome*)malloc(sizeof(struct chromosome));

	/* allocate memory for nodes */
	chromoNew->nodes = (struct node**)malloc(chromo->numNodes * sizeof(struct node*));

	/* allocate memory for outputNodes matrix */
	chromoNew->outputNodes = (int*)malloc(chromo->numOutputs * sizeof(int));

	/* allocate memory for active nodes matrix */
	chromoNew->activeNodes = (int*)malloc(chromo->numNodes * sizeof(int));

    /* allocate memory for smNodes matrix */
    chromoNew->smNodes = (int*)malloc(chromo->numNodes * sizeof(int));

	/* allocate memory for chromosome outputValues */
	chromoNew->outputValues = (double*)malloc(chromo->numOutputs * sizeof(double));

	/* Initialise each of the chromosomes nodes */
	for (i = 0; i < chromo->numNodes; i++) {
		chromoNew->nodes[i] = initialiseNode(chromo->numInputs, chromo->numNodes, chromo->arity, chromo->funcSet->numFunctions, 1.0, chromo->recurrentConnectionProbability, i, chromo->smNodes);
		copyNode(chromoNew->nodes[i], chromo->nodes[i]);
	}

    /* set each of the chromosomes smNode */
    for (i = 0; i < chromo->numNodes; i++){
        chromoNew->smNodes[i] = chromo->smNodes[i];
    }

	/* set each of the chromosomes outputs */
	for (i = 0; i < chromo->numOutputs; i++) {
		chromoNew->outputNodes[i] = chromo->outputNodes[i];
	}

	/* set the number of inputs, nodes and outputs */
	chromoNew->numInputs = chromo->numInputs;
	chromoNew->numNodes = chromo->numNodes;
	chromoNew->numOutputs = chromo->numOutputs;
	chromoNew->arity = chromo->arity;
    chromoNew->numToDos = chromo->numToDos;
	chromoNew->recurrentConnectionProbability = chromo->recurrentConnectionProbability;


    for(i = 0; i < TODO_LENGTH; i++)
        chromoNew->todoList[i] = chromo->todoList[i];

	/* copy over the chromsosme fitness */
	chromoNew->fitness = chromo->fitness;

	/* copy over the number of gnerations to find a solution */
	chromoNew->generation = chromo->generation;

	/* copy over the functionset */
	chromoNew->funcSet = (struct functionSet*)malloc(sizeof(struct functionSet));
	copyFunctionSet(chromoNew->funcSet, chromo->funcSet);

	/* set the active nodes in the newly generated chromosome */
	setChromosomeActiveNodes(chromoNew);

	/* used internally by execute chromosome */
	chromoNew->nodeInputsHold = (double*)malloc(chromo->arity * sizeof(double));


	return chromoNew;
}


/*
	Frees the memory associated with the given chromosome structure
*/
DLL_EXPORT void freeChromosome(struct chromosome *chromo) {

	int i;

	/* attempt to prevent user double freeing */
	if (chromo == NULL) {
		printf("Warning: double freeing of chromosome prevented.\n");
		return;
	}

	for (i = 0; i < chromo->numNodes; i++) {
		freeNode(chromo->nodes[i]);
	}

	free(chromo->nodeInputsHold);
	free(chromo->funcSet);
    free(chromo->smNodes);
	free(chromo->activeNodes);
	free(chromo->outputValues);
	free(chromo->outputNodes);
	free(chromo->nodes);
	free(chromo);
	chromo = NULL;
}



/*
	Prints the given chromosome to the screen
*/
DLL_EXPORT void printChromosome(struct chromosome *chromo, int weights) {

	int i, j;

	/* error checking */
	if (chromo == NULL) {
		printf("Error: chromosome has not been initialised and cannot be printed.\n");
		return;
	}

	/* set the active nodes in the given chromosome */
	setChromosomeActiveNodes(chromo);

	/* for all the chromo inputs*/
	for (i = 0; i < chromo->numInputs; i++) {
		printf("(%d):\tinput\n", i);
	}

	/* for all the hidden nodes */
	for (i = 0; i < chromo->numNodes; i++) {

		/* print the node function */
		printf("(%d):\t%s\t", chromo->numInputs + i, chromo->funcSet->functionNames[chromo->nodes[i]->function]);

		/* for the arity of the node */
		for (j = 0; j < getChromosomeNodeArity(chromo, i); j++) {

			/* print the node input information */
			if (weights == 1) {
				printf("%d,%+.1f\t", chromo->nodes[i]->inputs[j], chromo->nodes[i]->weights[j]);
			}
			else {
				printf("%d ", chromo->nodes[i]->inputs[j]);
			}
		}

		/* Highlight active nodes */
		if (chromo->nodes[i]->active == 1) {
			printf("*");
		}

        printf("\t[");

        for (j = 0; j < NUM_ARGUMENTS; j++){
            printf("% 02d, ", chromo->nodes[i]->arguments[j]);
        }

		printf("]\t");

        if(weights == 2){
            printf("%+.2f\t", chromo->nodes[i]->output);

        }
        printf("\n");
	}

	/* for all of the outputs */
	printf("outputs: ");
	for (i = 0; i < chromo->numOutputs; i++) {

		/* print the output node locations */
		printf("%d ", chromo->outputNodes[i]);
	}

	printf("\n\n");
}



/*
	Executes the given chromosome
*/
DLL_EXPORT struct chromosome* executeChromosome(struct chromosome *chromo, const double *inputs) {

	int i, j;
	int nodeInputLocation;
	int currentActiveNode;
	int currentActiveNodeFunction;
	int nodeArity;
    int smActivations = 0;

    struct chromosome* newChromo = NULL;

	const int numInputs = chromo->numInputs;
    //const int numNodes = chromo->numNodes;
	const int numActiveNodes = chromo->numActiveNodes;
	const int numOutputs = chromo->numOutputs;

	/* error checking */
	if (chromo == NULL) {
		printf("Error: cannot execute uninitialised chromosome.\n Terminating CGP-Library.\n");
		exit(0);
	}

    //checkChromosomeInputs(chromo);

    /* for all of the active nodes */
	for (i = 0; i < numActiveNodes; i++) {

		/* get the index of the current active node */
		currentActiveNode = chromo->activeNodes[i];

		/* get the arity of the current node */
		nodeArity = chromo->nodes[currentActiveNode]->actArity;

		/* for each of the active nodes inputs */
		for (j = 0; j < nodeArity; j++) {

			/* gather the nodes input locations */
			nodeInputLocation = chromo->nodes[currentActiveNode]->inputs[j];

			if (nodeInputLocation < numInputs) {
                //printf("inpuit %d:%f\n",nodeInputLocation, inputs[nodeInputLocation]);
				chromo->nodeInputsHold[j] = inputs[nodeInputLocation];
			}
			else {
				chromo->nodeInputsHold[j] = chromo->nodes[nodeInputLocation - numInputs]->output;
			}
		}

        //TODO:handle todo
        //addTodo(chromo, currentActiveNode,)

		/* get the functionality of the active node under evaluation */
		currentActiveNodeFunction = chromo->nodes[currentActiveNode]->function;

        if(chromo->funcSet->type[currentActiveNodeFunction] == COMP_NODE) {
			/* calculate the output of the active node under evaluation */
			chromo->nodes[currentActiveNode]->output = chromo->funcSet->functions[currentActiveNodeFunction](nodeArity, chromo->nodeInputsHold, chromo->nodes[currentActiveNode]->weights);
            //if(1 == 1){//isinf(chromo->nodes[currentActiveNode]->output) == 1) {

                //printf("a%d, f:%d, v:%f, i%d:%f, i%d:%f \n", currentActiveNode, currentActiveNodeFunction, chromo->nodes[currentActiveNode]->output, \
                chromo->nodes[currentActiveNode]->inputs[0], chromo->nodeInputsHold[0], \
                chromo->nodes[currentActiveNode]->inputs[1], chromo->nodeInputsHold[1]);//, \
                chromo->nodes[currentActiveNode]->inputs[2], chromo->nodeInputsHold[2]);

            //}

		}else if(chromo->funcSet->type[currentActiveNodeFunction] == INFO_NODE){
            chromo->nodes[currentActiveNode]->output = chromo->funcSet->infoFunctions[currentActiveNodeFunction](chromo,currentActiveNode, 1.0); //TODO: Add info weight parameter

        }else if(chromo->funcSet->type[currentActiveNodeFunction] == SM_NODE){
            /* if sm-node is activated */
            if(smActivations < 4 /*params->numSmActivations*/ && chromo->funcSet->functions[currentActiveNodeFunction](nodeArity, chromo->nodeInputsHold, chromo->nodes[currentActiveNode]->weights) == 1.0){

                chromo->todoList[chromo->numToDos].function = currentActiveNodeFunction;
                for(j = 0; j<NUM_ARGUMENTS;j++)
                    chromo->todoList[chromo->numToDos].arguments[j] = chromo->nodes[currentActiveNode]->arguments[j];
                chromo->todoList[chromo->numToDos].position = currentActiveNode;

                chromo->numToDos++;
                smActivations++;


                //newChromo = chromo->funcSet->smFunctions[currentActiveNodeFunction](chromo,currentActiveNode,chromo->nodes[currentActiveNode]->arguments,params);
                //chromo->todoList[chromo->numToDos]->function =

                //smActivations++;
//                break;
                //freeChromosome(chromo);
                //chromo = newChromo;
                //printf("%s ACTIVATED node:%d args:%d,%d,%d\n", chromo->funcSet->functionNames[currentActiveNodeFunction], currentActiveNode+chromo->numInputs, chromo->nodes[currentActiveNode]->arguments[0],chromo->nodes[currentActiveNode]->arguments[1],chromo->nodes[currentActiveNode]->arguments[2]);

            }
        }



		/* deal with doubles becoming NAN */
//		if (isnan(chromo->nodes[currentActiveNode]->output) != 0) {
//			chromo->nodes[currentActiveNode]->output = 0;
//		}
//
//		/* prevent double form going to inf and -inf */
//		else if (isinf(chromo->nodes[currentActiveNode]->output) != 0 ) {
//
//			if (chromo->nodes[currentActiveNode]->output > 0) {
//				chromo->nodes[currentActiveNode]->output = DBL_MAX;
//			}
//			else {
//				chromo->nodes[currentActiveNode]->output = DBL_MIN;
//			}
//		}

//        if(currentActiveNode == 1){
//            if(chromo->nodes[currentActiveNode]->output>1000.0){
//
//                printf("%f\n", chromo->nodes[currentActiveNode]->output);
//            }
//        }
	}


    for (i = 0; i<chromo->numToDos; i++){
        newChromo = chromo->funcSet->smFunctions[chromo->todoList[i].function](chromo,chromo->todoList[i].position,chromo->todoList[i].arguments);
        freeChromosome(chromo);
        chromo = newChromo;
    }

    chromo->numToDos = 0;


  //  if(newChromo != NULL){
        //copyChromosome(chromo, newChromo);
   //     freeChromosome(chromo);
     //   chromo = newChromo;
//        if(newChromo == chromo){
//            //printf("Chromo not changed\n");
//        }else{
//            //printf("Exec finished, Chromo updated! Nodes changed: %d -> %d\n", chromo->numNodes, newChromo->numNodes);
//           // printChromosome(chromo, 0);
//           // printChromosome(chromo, 0);
//        }
 //   }

    /* Set the chromosome outputs */
    for (i = 0; i < numOutputs; i++) {

        if (chromo->outputNodes[i] < numInputs) {
            chromo->outputValues[i] = inputs[chromo->outputNodes[i]];
        }
        else {
            chromo->outputValues[i] = chromo->nodes[chromo->outputNodes[i] - numInputs]->output;
        }
    }


    return chromo;
//    for (i = 0; i < numNodes; i++){
//        currentActiveNode = chromo->smNodes[i];
//    }
}

/*
	used to access the chromosome outputs after executeChromosome
	has been called
*/
DLL_EXPORT double getChromosomeOutput(struct chromosome *chromo, int output) {

	if (output < 0 || output > chromo->numOutputs) {
		printf("Error: output less than or greater than the number of chromosome outputs. Called from getChromosomeOutput.\n");
		exit(0);
	}

    double out = chromo->outputValues[output];
    if(out>1000000.0 || out < -1000000.0)
        out = -0.0;
	return out;
}



/*
	used to access the chromosome node values after executeChromosome
	has been called
*/
DLL_EXPORT double getChromosomeNodeValue(struct chromosome *chromo, int node) {
	if (node < 0 || node >= chromo->numNodes) {
		return NAN;
		printf("Error: node less than or greater than the number of nodes  in chromosome. Called from getChromosomeNodeValue.\n");
		exit(0);
	}

	return chromo->nodes[node]->output;
}


/*
	returns whether the specified node is active in the given chromosome
*/
DLL_EXPORT int isNodeActive(struct chromosome *chromo, int node) {

	if (node < 0 || node > chromo->numNodes) {
		printf("Error: node less than or greater than the number of nodes  in chromosome. Called from isNodeActive.\n");
		exit(0);
	}

	return chromo->nodes[node]->active;
}


/*
	Saves the given chromosome in a form which can be read in later
*/
DLL_EXPORT void saveChromosome(struct chromosome *chromo, char const *fileName) {

	int i, j;
	FILE *fp;

	/* create the chromosome file */
	fp = fopen(fileName, "w");

	/* ensure that the file was created correctly */
	if (fp == NULL) {
		printf("Warning: cannot save chromosome to '%s'. Chromosome was not saved.\n", fileName);
		return;
	}

	/* save meta information */
	fprintf(fp, "numInputs,%d\n", chromo->numInputs);
	fprintf(fp, "numNodes,%d\n", chromo->numNodes);
	fprintf(fp, "numOutputs,%d\n", chromo->numOutputs);
	fprintf(fp, "arity,%d\n", chromo->arity);

	fprintf(fp, "functionSet");

	for (i = 0; i < chromo->funcSet->numFunctions; i++) {
		fprintf(fp, ",%s", chromo->funcSet->functionNames[i]);
	}
	fprintf(fp, "\n");

	/* save the chromosome structure */
	for (i = 0; i < chromo->numNodes; i++) {

		fprintf(fp, "%d\n", chromo->nodes[i]->function);

		for (j = 0; j < chromo->arity; j++) {
			fprintf(fp, "%d,%f\n", chromo->nodes[i]->inputs[j], chromo->nodes[i]->weights[j]);
		}

        for (j = 0; j < NUM_ARGUMENTS; j++) {
            fprintf(fp, "%d\n", chromo->nodes[i]->arguments[j]);
        }
	}

	for (i = 0; i < chromo->numOutputs; i++) {
		fprintf(fp, "%d,", chromo->outputNodes[i]);
	}

	fclose(fp);
}


/*
	save the given chromosome to a graphviz .dot file
	(www.graphviz.org/‎)
*/
DLL_EXPORT void saveChromosomeDot(struct chromosome *chromo, int weights, char const *fileName) {

	int i, j;
	FILE *fp;

	char colour[20];
	char weight[20];

	fp = fopen(fileName, "w");

	if (fp == NULL) {
		return;
	}

	/* */
	fprintf(fp, "digraph NeuralNetwork {\n");

	/* landscape, square and centre */
	fprintf(fp, "rankdir=LR;\n");
	fprintf(fp, "size=\"4,3\";\n");
	fprintf(fp, "center = true;\n");

	/* for all the inputs */
	for (i = 0; i < getNumChromosomeInputs(chromo); i++) {

		fprintf(fp, "node%d [label=\"(%d) Input\", color=black, labelfontcolor=black, fontcolor=black];\n", i, i);
	}

	/* for all nodes */
	for (i = 0; i < getNumChromosomeNodes(chromo); i++) {

		if (chromo->nodes[i]->active == 1) {
			strncpy(colour, "black", 20);
		}
		else {
			strncpy(colour, "lightgrey", 20);
		}
		fprintf(fp, "node%d [label=\"(%d) %s\", color=%s, labelfontcolor=%s, fontcolor=%s];\n", i + getNumChromosomeInputs(chromo), i + getNumChromosomeInputs(chromo), chromo->funcSet->functionNames[chromo->nodes[i]->function], colour, colour, colour);

		/* for each node input */
		for (j = 0; j < getChromosomeNodeArity(chromo, i); j++) {

			if (weights == 1) {
				snprintf(weight, 20, "%.2f", chromo->nodes[i]->weights[j]);
			}
			else {
				snprintf(weight, 20, " (%d)", j);
			}


			fprintf(fp, "node%d -> node%d [label=\"%s\", labelfontcolor=%s, fontcolor=%s, bold=true, color=%s];\n", chromo->nodes[i]->inputs[j], i + getNumChromosomeInputs(chromo), weight, colour, colour, colour);
		}
	}

	for (i = 0; i < getNumChromosomeOutputs(chromo); i++) {

		fprintf(fp, "node%d [label=\"Output %d\", color=black, labelfontcolor=black, fontcolor=black];\n", i + getNumChromosomeInputs(chromo) + getNumChromosomeNodes(chromo), i);

		fprintf(fp, "node%d -> node%d [labelfontcolor=black, fontcolor=black, bold=true, color=black];\n", chromo->outputNodes[i], i + getNumChromosomeInputs(chromo) + getNumChromosomeNodes(chromo));
	}


	/* place inputs  on same line */
	fprintf(fp, "{ rank = source;");

	for (i = 0; i < getNumChromosomeInputs(chromo); i++) {
		fprintf(fp, " \"node%d\";", i);
	}
	fprintf(fp, " }\n");


	/* place outputs  on same line */
	fprintf(fp, "{ rank = max;");

	for (i = 0; i < getNumChromosomeOutputs(chromo); i++) {
		fprintf(fp, "\"node%d\";", i + getNumChromosomeInputs(chromo) + getNumChromosomeNodes(chromo));
	}
	fprintf(fp, " }\n");


	/* last line of dot file */
	fprintf(fp, "}");

	fclose(fp);
}


/*
	save the given chromosome as a latex equation

	Only compatible with feed-forward networks
	Only fully compatible with custom node functions
*/
DLL_EXPORT void saveChromosomeLatex(struct chromosome *chromo, int weights, char const *fileName) {

	int output;
	int i;
	FILE *fp;

	fp = fopen(fileName, "w");

	if (fp == NULL) {
		return;
	}

	/* document header */
	fprintf(fp, "\\documentclass{article}\n");
	fprintf(fp, "\\begin{document}\n");

	for (output = 0; output < chromo->numOutputs; output++) {

		fprintf(fp, "\\begin{equation}\n");

		/* function inputs */
		if (chromo->numInputs == 0) {
			fprintf(fp, "f()=");
		}
		else {

			fprintf(fp, "f_%d(x_0", output);

			for (i = 1; i < chromo->numInputs; i++) {

				fprintf(fp, ",x_%d", i);
			}

			fprintf(fp, ")=");
		}

		saveChromosomeLatexRecursive(chromo, chromo->outputNodes[output], fp);

		fprintf(fp, "\n\\end{equation}");
	}


	/* document footer */
	fprintf(fp, "\n\\end{document}");

	fclose(fp);
}

/*
	used by saveChromosomeLatex
*/
static void saveChromosomeLatexRecursive(struct chromosome *chromo, int index, FILE *fp) {

	int i;

	if (index < chromo->numInputs) {
		fprintf(fp, "x_%d", index);
		return;
	}

	/* add */
	if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "add", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, "\\left(");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		for (i = 1; i < getChromosomeNodeArity(chromo, index - chromo->numInputs); i++) {

			fprintf(fp, " + ");

			saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[i], fp);
		}

		fprintf(fp, "\\right)");
	}


	/* sub */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "sub", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, "\\left(");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		for (i = 1; i < getChromosomeNodeArity(chromo, index - chromo->numInputs); i++) {

			fprintf(fp, " - ");

			saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[i], fp);
		}

		fprintf(fp, "\\right)");
	}

	/* mul */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "mul", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, "\\left(");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		for (i = 1; i < getChromosomeNodeArity(chromo, index - chromo->numInputs); i++) {

			fprintf(fp, " \\times ");

			saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[i], fp);
		}

		fprintf(fp, "\\right)");
	}

	/* div (change to frac)*/
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "div", FUNCTIONNAMELENGTH) == 0 ) {

		if (getChromosomeNodeArity(chromo, index - chromo->numInputs) == 1) {
			saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);
		}
		else {

			for (i = 0; i < getChromosomeNodeArity(chromo, index - chromo->numInputs); i++) {

				if (i + 1 < getChromosomeNodeArity(chromo, index - chromo->numInputs)) {
					fprintf(fp, "\\frac{");
					saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[i], fp);
					fprintf(fp, "}{");
				}
				else if (i + 1 == getChromosomeNodeArity(chromo, index - chromo->numInputs) && getChromosomeNodeArity(chromo, index - chromo->numInputs) > 2) {
					saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[i], fp);
					fprintf(fp, "}}");
				}
				else {
					saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[i], fp);
					fprintf(fp, "}");
				}
			}
		}
	}

	/* abs */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "abs", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " \\left|");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		fprintf(fp, " \\right|");

	}

	/* sqrt */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "sqrt", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " \\sqrt{");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		fprintf(fp, " }");

	}


	/* sq */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "sq", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " (");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		fprintf(fp, " )^2");

	}

	/* cube */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "cube", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " (");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		fprintf(fp, " )^3");

	}

	/* exp */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "exp", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " e^{");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		fprintf(fp, " }");

	}

	/* sin */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "sin", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, "\\sin(");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		fprintf(fp, " )");

	}

	/* cos */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "cos", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " \\cos(");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		fprintf(fp, " )");

	}

	/* tan */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "tan", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " \\tan(");

		saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[0], fp);

		fprintf(fp, " )");

	}

	/* rand */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "rand", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " rand()");
	}

	/* pi */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "pi", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, "\\pi");
	}

	/* 0 */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "0", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " 0");
	}

	/* 1 */
	else if (strncmp(chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function], "1", FUNCTIONNAMELENGTH) == 0 ) {

		fprintf(fp, " 1");
	}

	/* other */
	else {

		fprintf(fp, "%s(", chromo->funcSet->functionNames[chromo->nodes[index - chromo->numInputs]->function]);

		for (i = 0; i < getChromosomeNodeArity(chromo, index - chromo->numInputs); i++) {

			saveChromosomeLatexRecursive(chromo, chromo->nodes[index - chromo->numInputs]->inputs[i], fp);

			if (i < getChromosomeNodeArity(chromo, index - chromo->numInputs) - 1)
				fprintf(fp, ", ");
		}

		fprintf(fp, ")");
	}

}

DLL_EXPORT int compareChromosomes(struct chromosome *chromoA, struct chromosome *chromoB) {

	int i, j;

	/* ensure that the chromosomes don't point to NULL */
	if (chromoA == NULL || chromoB == NULL) {
		return 0;
	}

	/* Check the high level parameters */
	if (chromoA->numInputs != chromoB->numInputs) {
		return 0;
	}

	if (chromoA->numNodes != chromoB->numNodes) {
		return 0;
	}

	if (chromoA->numOutputs != chromoB->numOutputs) {
		return 0;
	}

	if (chromoA->arity != chromoB->arity) {
		return 0;
	}

	/* for each node*/
	for (i = 0; i < chromoA->numNodes; i++) {

		/* Check the function genes */
		if (chromoA->nodes[i]->function != chromoB->nodes[i]->function) {
			return 0;
		}

		/* for each node input */
		for (j = 0; j < chromoA->arity; j++) {

			/* Check the node inputs */
			if (chromoA->nodes[i]->inputs[j] != chromoB->nodes[i]->inputs[j]) {
				return 0;
			}
		}
	}

	/* for all of the outputs */
	for (i = 0; i < chromoA->numOutputs; i++) {

		/* Check the outputs */
		if (chromoA->outputNodes[i] != chromoB->outputNodes[i] ) {
			return 0;
		}
	}

	return 1;
}


DLL_EXPORT int compareChromosomesANN(struct chromosome *chromoA, struct chromosome *chromoB) {

	int i, j;

	/* ensure that the chromosomes don't point to NULL */
	if (chromoA == NULL || chromoB == NULL) {
		return 0;
	}

	/* Check the high level parameters */
	if (chromoA->numInputs != chromoB->numInputs) {
		return 0;
	}

	if (chromoA->numNodes != chromoB->numNodes) {
		return 0;
	}

	if (chromoA->numOutputs != chromoB->numOutputs) {
		return 0;
	}

	if (chromoA->arity != chromoB->arity) {
		return 0;
	}

	/* for each node*/
	for (i = 0; i < chromoA->numNodes; i++) {

		/* Check the function genes */
		if (chromoA->nodes[i]->function != chromoB->nodes[i]->function) {
			return 0;
		}

		/* for each node input */
		for (j = 0; j < chromoA->arity; j++) {

			/* Check the node inputs */
			if (chromoA->nodes[i]->inputs[j] != chromoB->nodes[i]->inputs[j]) {
				return 0;
			}

			/* Check the connection weights inputs */
			if (chromoA->nodes[i]->weights[j] != chromoB->nodes[i]->weights[j]) {
				return 0;
			}
		}
	}

	/* for all of the outputs */
	for (i = 0; i < chromoA->numOutputs; i++) {

		/* Check the outputs */
		if (chromoA->outputNodes[i] != chromoB->outputNodes[i] ) {
			return 0;
		}
	}

	return 1;
}






DLL_EXPORT int compareChromosomesActiveNodes(struct chromosome *chromoA, struct chromosome *chromoB) {

	int i, j;

	/* ensure that the chromosomes don't point to NULL */
	if (chromoA == NULL || chromoB == NULL) {
		return 0;
	}

	/* Check the high level parameters */
	if (chromoA->numInputs != chromoB->numInputs) {
		return 0;
	}

	if (chromoA->numNodes != chromoB->numNodes) {
		return 0;
	}

	if (chromoA->numOutputs != chromoB->numOutputs) {
		return 0;
	}

	if (chromoA->arity != chromoB->arity) {
		return 0;
	}

	/* for each node*/
	for (i = 0; i < chromoA->numNodes; i++) {

		/* if the node is active in both chromosomes */
		if (chromoA->nodes[i]->active == 1 && chromoB->nodes[i]->active == 1) {

			/* Check the function genes */
			if (chromoA->nodes[i]->function != chromoB->nodes[i]->function) {
				return 0;
			}

			/* for each node input */
			for (j = 0; j < chromoA->arity; j++) {

				/* Check the node inputs */
				if (chromoA->nodes[i]->inputs[j] != chromoB->nodes[i]->inputs[j]) {
					return 0;
				}

				/* Check the connection weights inputs */
				if (chromoA->nodes[i]->weights[j] != chromoB->nodes[i]->weights[j]) {
					return 0;
				}
			}
		}
		/* if the node is active in one chromosome */
		else if (chromoA->nodes[i]->active != chromoB->nodes[i]->active ) {
			return 0;
		}

		/* The node is inactive in both chromosomes */
		else {
			/* do nothing */
		}
	}

	/* for all of the outputs */
	for (i = 0; i < chromoA->numOutputs; i++) {

		/* Check the outputs */
		if (chromoA->outputNodes[i] != chromoB->outputNodes[i] ) {
			return 0;
		}
	}

	return 1;
}


DLL_EXPORT int compareChromosomesActiveNodesANN(struct chromosome *chromoA, struct chromosome *chromoB) {

	int i, j;

	/* ensure that the chromosomes don't point to NULL */
	if (chromoA == NULL || chromoB == NULL) {
		return 0;
	}

	/* Check the high level parameters */
	if (chromoA->numInputs != chromoB->numInputs) {
		return 0;
	}

	if (chromoA->numNodes != chromoB->numNodes) {
		return 0;
	}

	if (chromoA->numOutputs != chromoB->numOutputs) {
		return 0;
	}

	if (chromoA->arity != chromoB->arity) {
		return 0;
	}

	/* for each node*/
	for (i = 0; i < chromoA->numNodes; i++) {

		/* if the node is active in both chromosomes */
		if (chromoA->nodes[i]->active == 1 && chromoB->nodes[i]->active == 1) {

			/* Check the function genes */
			if (chromoA->nodes[i]->function != chromoB->nodes[i]->function) {
				return 0;
			}

			/* for each node input */
			for (j = 0; j < chromoA->arity; j++) {

				/* Check the node inputs */
				if (chromoA->nodes[i]->inputs[j] != chromoB->nodes[i]->inputs[j]) {
					return 0;
				}
			}
		}
		/* if the node is active in one chromosome */
		else if (chromoA->nodes[i]->active != chromoB->nodes[i]->active ) {
			return 0;
		}

		/* The node is inactive in both chromosomes */
		else {
			/* do nothing */
		}
	}

	/* for all of the outputs */
	for (i = 0; i < chromoA->numOutputs; i++) {

		/* Check the outputs */
		if (chromoA->outputNodes[i] != chromoB->outputNodes[i] ) {
			return 0;
		}
	}

	return 1;
}




/*
	Mutates the given chromosome using the mutation method described in parameters
*/
DLL_EXPORT void mutateChromosome(struct parameters *params, struct chromosome *chromo) {

	params->mutationType(params, chromo);

	setChromosomeActiveNodes(chromo);
}


/*
	removes the inactive nodes from the given chromosome
*/
DLL_EXPORT void removeInactiveNodes(struct chromosome *chromo) {

	int i, j, k;

	int originalNumNodes = chromo->numNodes;

	/* set the active nodes */
	setChromosomeActiveNodes(chromo);

	/* for all nodes */
	for (i = 0; i < chromo->numNodes - 1; i++) {

		/* if the node is inactive */
		if (chromo->nodes[i]->active == 0) {

			/* set the node to be the next node */
			for (j = i; j < chromo->numNodes - 1; j++) {
				copyNode(chromo->nodes[j], chromo->nodes[j + 1]);
			}

			/* */
			for (j = 0; j < chromo->numNodes; j++) {
				for (k = 0; k < chromo->arity; k++) {

					if (chromo->nodes[j]->inputs[k] >= i + chromo->numInputs) {
						chromo->nodes[j]->inputs[k]--;
					}
				}
			}

			/* for the number of chromosome outputs */
			for (j = 0; j < chromo->numOutputs; j++) {

				if (chromo->outputNodes[j] >= i + chromo->numInputs) {
					chromo->outputNodes[j]--;
				}
			}

			/* de-increment the number of nodes */
			chromo->numNodes--;

			/* made the newly assigned node be evaluated */
			i--;
		}
	}

	for (i = chromo->numNodes; i < originalNumNodes; i++) {
		freeNode(chromo->nodes[i]);
	}

	if (chromo->nodes[chromo->numNodes - 1]->active == 0) {
		freeNode(chromo->nodes[chromo->numNodes - 1]);
		chromo->numNodes--;
	}

	/* reallocate the memory associated with the chromosome */
	chromo->nodes = (struct node**)realloc(chromo->nodes, chromo->numNodes * sizeof(struct node*));
	chromo->activeNodes = (int*)realloc(chromo->activeNodes, chromo->numNodes * sizeof(int));

	/* set the active nodes */
	setChromosomeActiveNodes(chromo);
}


/*
	sets the fitness of the given chromosome
*/
DLL_EXPORT void setChromosomeFitness(struct parameters *params, struct chromosome *chromo, struct dataSet *data) {

	double fitness;

	setChromosomeActiveNodes(chromo);

	resetChromosome(chromo);

    //printChromosome(chromo, 0);

	fitness = params->fitnessFunction(params, chromo, data);

	chromo->fitness = fitness;
}


/*
	reset the output values of all chromosome nodes to zero
*/
DLL_EXPORT void resetChromosome(struct chromosome *chromo) {

	int i;

	for (i = 0; i < chromo->numNodes; i++) {
		chromo->nodes[i]->output = 0;
	}
}

/*
	copies the contents of one chromosome to another. Provided the number of inputs, nodes, outputs and node arity are the same.
*/
DLL_EXPORT void copyChromosome(struct chromosome *chromoDest, struct chromosome *chromoSrc) {

	int i;

	/* error checking  */
	if (chromoDest->numInputs != chromoSrc->numInputs) {
		printf("Error: cannot copy a chromosome to a chromosome of different dimensions. The number of chromosome inputs do not match.\n");
		printf("Terminating CGP-Library.\n");
		exit(0);
	}

	if (chromoDest->numNodes != chromoSrc->numNodes) {
		printf("Error: cannot copy a chromosome to a chromosome of different dimensions. The number of chromosome nodes do not match.\n");
		printf("Terminating CGP-Library.\n");
		exit(0);
	}

	if (chromoDest->numOutputs != chromoSrc->numOutputs) {
		printf("Error: cannot copy a chromosome to a chromosome of different dimensions. The number of chromosome outputs do not match.\n");
		printf("Terminating CGP-Library.\n");
		exit(0);
	}

	if (chromoDest->arity != chromoSrc->arity) {
		printf("Error: cannot copy a chromosome to a chromosome of different dimensions. The arity of the chromosome nodes do not match.\n");
		printf("Terminating CGP-Library.\n");
		exit(0);
	}

	/* copy nodes and which are active */
	for (i = 0; i < chromoSrc->numNodes; i++) {
		copyNode(chromoDest->nodes[i],  chromoSrc->nodes[i]);
		chromoDest->activeNodes[i] = chromoSrc->activeNodes[i];
	}

	/* copy functionset */
	copyFunctionSet(chromoDest->funcSet, chromoSrc->funcSet);

	/* copy each of the chromosomes outputs */
	for (i = 0; i < chromoSrc->numOutputs; i++) {
		chromoDest->outputNodes[i] = chromoSrc->outputNodes[i];
	}

	/* copy the number of active node */
	chromoDest->numActiveNodes = chromoSrc->numActiveNodes;

	/* copy the fitness */
	chromoDest->fitness = chromoSrc->fitness;

	/* copy generation */
	chromoDest->generation = chromoSrc->generation;
}

/*
	Gets the number of chromosome inputs
*/
DLL_EXPORT int getNumChromosomeInputs(struct chromosome *chromo) {
	return chromo->numInputs;
}

/*
	Gets the number of chromosome nodes
*/
DLL_EXPORT int getNumChromosomeNodes(struct chromosome *chromo) {
	return chromo->numNodes;
}

/*
	Gets the number of chromosome active nodes
*/
DLL_EXPORT int getNumChromosomeActiveNodes(struct chromosome *chromo) {
	return chromo->numActiveNodes;
}

/*
	Gets the number of chromosome outputs
*/
DLL_EXPORT int getNumChromosomeOutputs(struct chromosome *chromo) {
	return chromo->numOutputs;
}

/*
	Gets the chromosome node arity
*/
DLL_EXPORT int getChromosomeNodeArity(struct chromosome *chromo, int index) {

	int chromoArity = chromo->arity;
	int maxArity = chromo->funcSet->maxNumInputs[chromo->nodes[index]->function];

	if (maxArity == -1) {
		return chromoArity;
	}
	else if (maxArity < chromoArity) {
		return maxArity;
	}
	else {
		return chromoArity;
	}
}

/*
	Gets the chromosome fitness
*/
DLL_EXPORT double getChromosomeFitness(struct chromosome *chromo) {
	return chromo->fitness;
}

/*
	Gets the number of active connections in the given chromosome
*/
DLL_EXPORT int getNumChromosomeActiveConnections(struct chromosome *chromo) {

	int i;
	int complexity = 0;

	for (i = 0; i < chromo->numActiveNodes; i++) {
		complexity += chromo->nodes[chromo->activeNodes[i]]->actArity;
	}

	return complexity;
}

/*
	Gets the number of generations required to find the given chromosome
*/
DLL_EXPORT int getChromosomeGenerations(struct chromosome *chromo) {
	return chromo->generation;
}


/*
	set the active nodes in the given chromosome
*/
void setChromosomeActiveNodes(struct chromosome *chromo) {

	int i,j;

	/* error checking */
	if (chromo == NULL) {
		printf("Error: chromosome has not been initialised and so the active nodes cannot be set.\n");
		return;
	}

	/* set the number of active nodes to zero */
	chromo->numActiveNodes = 0;

    /* reset the active nodes */
	for (i = 0; i < chromo->numNodes; i++) {
        //printf("%s\n",chromo->funcSet->functionNames[chromo->nodes[i]->function]);
        //todo: REMOVEHOTFIX: SET ALL ACTIVE AND RETURN!
        //chromo->nodes[i]->active = 1;
		/* log the node as active */
		chromo->nodes[i]->active = 1;
		chromo->activeNodes[chromo->numActiveNodes] = i;
		chromo->numActiveNodes++;
	}

	/* start the recursive search for active nodes from the output nodes for the number of output nodes */
//	for (i = 0; i < chromo->numOutputs; i++) {
//
//		/* if the output connects to a chromosome input, skip */
//		if (chromo->outputNodes[i] < chromo->numInputs) {
//			continue;
//		}
//
//		/* begin a recursive search for active nodes */
//		recursivelySetActiveNodes(chromo, chromo->outputNodes[i]);
//	}

    /* start the recursive search for SM nodes */
//    for (i = 0; i < chromo->numNodes; i++) {
//
//        /* ignore non SM nodes */
//        if (chromo->smNodes[i] == 0){
//            continue;
//        }
//
//        /* begin a recursive search for active nodes */
//        recursivelySetActiveNodes(chromo, i + chromo->numInputs);
//
//        /* set the nodes actual arity*/
//        chromo->nodes[i]->actArity = getChromosomeNodeArity(chromo, i);
//
//        /* recursively log all the nodes to which the current nodes connect as active */
//        for (j = 0; j < chromo->nodes[i]->actArity; j++) {
//            recursivelySetActiveNodes(chromo, chromo->nodes[i]->inputs[j]);
//        }
//    }


	/* place active nodes in order */
	sortIntArray(chromo->activeNodes, chromo->numActiveNodes);
}


/*
	used by setActiveNodes to recursively search for active nodes
*/
static void recursivelySetActiveNodes(struct chromosome *chromo, int nodeIndex) {

	int i;

	/* if the given node is an input, stop */
	if (nodeIndex < chromo->numInputs) {
		return;
	}

	if(nodeIndex >= chromo->numNodes + chromo->numInputs){
		fprintf(stderr, "ERROR: Input %d does not exist\n", nodeIndex);
		return;
	}


	/* if the given node has already been flagged as active */
	if (chromo->nodes[nodeIndex - chromo->numInputs]->active == 1) {
		return;
	}

	/* log the node as active */
	chromo->nodes[nodeIndex - chromo->numInputs]->active = 1;
	chromo->activeNodes[chromo->numActiveNodes] = nodeIndex - chromo->numInputs;
	chromo->numActiveNodes++;

	/* set the nodes actual arity*/
	chromo->nodes[nodeIndex - chromo->numInputs]->actArity = getChromosomeNodeArity(chromo, nodeIndex - chromo->numInputs);

	/* recursively log all the nodes to which the current nodes connect as active */
	for (i = 0; i < chromo->nodes[nodeIndex - chromo->numInputs]->actArity; i++) {
		recursivelySetActiveNodes(chromo, chromo->nodes[nodeIndex - chromo->numInputs]->inputs[i]);
	}
}


/*
	Sorts the given array of chromosomes by fitness, lowest to highest
	uses insertion sort (quickish and stable)
*/
static void sortChromosomeArray(struct chromosome **chromoArray, int numChromos) {

	int i, j;
	struct chromosome *chromoTmp;

	for (i = 0; i < numChromos; i++) {
		for (j = i; j < numChromos; j++) {

			if (chromoArray[i]->fitness > chromoArray[j]->fitness) {
				chromoTmp = chromoArray[i];
				chromoArray[i] = chromoArray[j];
				chromoArray[j] = chromoTmp;
			}
		}
	}
}



/*
	Dataset functions
*/


/*
	Initialises data structure and assigns values for given arrays
	arrays must take the form
	inputs[numSamples][numInputs]
	outputs[numSamples][numOutputs]
*/
DLL_EXPORT struct dataSet *initialiseDataSetFromArrays(int numInputs, int numOutputs, int numSamples, double *inputs, double *outputs) {

	int i, j;
	struct dataSet *data;

	/* initialise memory for data structure */
	data = (struct dataSet*)malloc(sizeof(struct dataSet));

	data->numInputs = numInputs;
	data->numOutputs = numOutputs;
	data->numSamples = numSamples;

	data->inputData = (double**)malloc(data->numSamples * sizeof(double*));
	data->outputData = (double**)malloc(data->numSamples * sizeof(double*));

	for (i = 0; i < data->numSamples; i++) {

		data->inputData[i] = (double*)malloc(data->numInputs * sizeof(double));
		data->outputData[i] = (double*)malloc(data->numOutputs * sizeof(double));

		for (j = 0; j < data->numInputs; j++) {
			data->inputData[i][j] = inputs[(i * data->numInputs) + j];
		}

		for (j = 0; j < data->numOutputs; j++) {
			data->outputData[i][j] = outputs[(i * data->numOutputs) + j];
		}
	}

	return data;
}


/*
	Initialises data structure and assigns values of given file
*/
DLL_EXPORT struct dataSet *initialiseDataSetFromFile(char const *file) {

	int i;
	struct dataSet *data;
	FILE *fp;
	char *line, *record;
	char buffer[1024];
	int lineNum = -1;
	int col;

	/* attempt to open the given file */
	fp = fopen(file, "r");

	/* if the file cannot be found */
	if (fp == NULL) {
		printf("Error: file '%s' cannot be found.\nTerminating CGP-Library.\n", file);
		exit(0);
	}

	/* initialise memory for data structure */
	data = (struct dataSet*)malloc(sizeof(struct dataSet));

	/* for every line in the given file */
	while ( (line = fgets(buffer, sizeof(buffer), fp)) != NULL) {

		/* deal with the first line containing meta data */
		if (lineNum == -1) {

			sscanf(line, "%d,%d,%d", &(data->numInputs), &(data->numOutputs), &(data->numSamples));

			data->inputData = (double**)malloc(data->numSamples * sizeof(double*));
			data->outputData = (double**)malloc(data->numSamples * sizeof(double*));

			for (i = 0; i < data->numSamples; i++) {
				data->inputData[i] = (double*)malloc(data->numInputs * sizeof(double));
				data->outputData[i] = (double*)malloc(data->numOutputs * sizeof(double));
			}
		}
		/* the other lines contain input output pairs */
		else {

			/* get the first value on the given line */
			record = strtok(line, " ,\n");
			col = 0;

			/* until end of line */
			while (record != NULL) {

				/* if its an input value */
				if (col < data->numInputs) {
					data->inputData[lineNum][col] = atof(record);
				}

				/* if its an output value */
				else {

					data->outputData[lineNum][col - data->numInputs] = atof(record);
				}

				/* get the next value on the given line */
				record = strtok(NULL, " ,\n");

				/* increment the current col index */
				col++;
			}
		}

		/* increment the current line index */
		lineNum++;
	}

	fclose(fp);

	return data;
}


/*
	frees given dataSet
*/
DLL_EXPORT void freeDataSet(struct dataSet *data) {

	int i;

	/* attempt to prevent user double freeing */
	if (data == NULL) {
		printf("Warning: double freeing of dataSet prevented.\n");
		return;
	}

	for (i = 0; i < data->numSamples; i++) {
		free(data->inputData[i]);
		free(data->outputData[i]);
	}

	free(data->inputData);
	free(data->outputData);
	free(data);
}


/*
	prints the given data structure to the screen
*/
DLL_EXPORT void printDataSet(struct dataSet *data) {

	int i, j;

	printf("DATA SET\n");
	printf("Inputs: %d, ", data->numInputs);
	printf("Outputs: %d, ", data->numOutputs);
	printf("Samples: %d\n", data->numSamples);

	for (i = 0; i < data->numSamples; i++) {

		for (j = 0; j < data->numInputs; j++) {
			printf("%f ", data->inputData[i][j]);
		}

		printf(" : ");

		for (j = 0; j < data->numOutputs; j++) {
			printf("%f ", data->outputData[i][j]);
		}

		printf("\n");
	}
}


/*
	saves dataset to file
*/
DLL_EXPORT void saveDataSet(struct dataSet *data, char const *fileName) {

	int i, j;
	FILE *fp;

	fp = fopen(fileName, "w");

	if (fp == NULL) {
		printf("Warning: cannot save data set to %s. Data set was not saved.\n", fileName);
		return;
	}

	fprintf(fp, "%d,", data->numInputs);
	fprintf(fp, "%d,", data->numOutputs);
	fprintf(fp, "%d,", data->numSamples);
	fprintf(fp, "\n");


	for (i = 0; i < data->numSamples; i++) {

		for (j = 0; j < data->numInputs; j++) {
			fprintf(fp, "%f,", data->inputData[i][j]);
		}

		for (j = 0; j < data->numOutputs; j++) {
			fprintf(fp, "%f,", data->outputData[i][j]);
		}

		fprintf(fp, "\n");
	}

	fclose(fp);
}


/*
	returns the number of inputs for each sample in the given dataSet
*/
DLL_EXPORT int getNumDataSetInputs(struct dataSet *data) {
	return data->numInputs;
}


/*
	returns the number of outputs for each sample in the given dataSet
*/
DLL_EXPORT int getNumDataSetOutputs(struct dataSet *data) {
	return data->numOutputs;
}


/*
	returns the number of samples in the given dataSet
*/
DLL_EXPORT int getNumDataSetSamples(struct dataSet *data) {
	return data->numSamples;
}


/*
	returns the inputs of the given sample of the given dataSet
*/
DLL_EXPORT double *getDataSetSampleInputs(struct dataSet *data, int sample) {
	return data->inputData[sample];
}


/*
	returns the given input of the given sample of the given dataSet
*/
DLL_EXPORT double getDataSetSampleInput(struct dataSet *data, int sample, int input) {
	return data->inputData[sample][input];
}


/*
	returns the outputs of the given sample of the given dataSet
*/
DLL_EXPORT double *getDataSetSampleOutputs(struct dataSet *data, int sample) {
	return data->outputData[sample];
}


/*
	returns the given output of the given sample of the given dataSet
*/
DLL_EXPORT double getDataSetSampleOutput(struct dataSet *data, int sample, int output) {
	return data->outputData[sample][output];
}



/*
	Results Functions
*/


/*
	initialises a results structure
*/
struct results* initialiseResults(struct parameters *params, int numRuns) {

	struct results *rels;

	rels = (struct results*)malloc(sizeof(struct results));
	rels->bestChromosomes = (struct chromosome**)malloc(numRuns * sizeof(struct chromosome*));

	rels->numRuns = numRuns;

	/*
		Initialised chromosomes are returns from runCGP and stored in a results structure.
		Therefore they should not be initialised here.
	*/

	return rels;
}


/*
	free a initialised results structure
*/
DLL_EXPORT void freeResults(struct results *rels) {

	int i;

	/* attempt to prevent user double freeing */
	if (rels == NULL) {
		printf("Warning: double freeing of results prevented.\n");
		return;
	}

	for (i = 0; i < rels->numRuns; i++) {
		freeChromosome(rels->bestChromosomes[i]);
	}

	free(rels->bestChromosomes);
	free(rels);
}


/*
	saves results structure to file
*/
DLL_EXPORT void saveResults(struct results *rels, char const *fileName) {

	FILE *fp;
	int i;

	struct chromosome *chromoTemp;

	if (rels == NULL) {
		printf("Warning: cannot save uninitialised results structure. Results not saved.\n");
		return;
	}

	fp = fopen(fileName, "w");

	if (fp == NULL) {
		printf("Warning: cannot open '%s' and so cannot save results to that file. Results not saved.\n", fileName);
		return;
	}

	fprintf(fp, "Run,Fitness,Generations,Active Nodes\n");

	for (i = 0; i < rels->numRuns; i++) {

		chromoTemp = getChromosome(rels, i);

		fprintf(fp, "%d,%f,%d,%d\n", i, chromoTemp->fitness, chromoTemp->generation, chromoTemp->numActiveNodes);

		freeChromosome(chromoTemp);
	}

	fclose(fp);
}


/*
	Gets the number of chromosomes in the results structure
*/
DLL_EXPORT int getNumChromosomes(struct results *rels) {
	return rels->numRuns;
}


/*
	returns the average number of chromosome active nodes from repeated
	run results specified in rels.
*/
DLL_EXPORT double getAverageActiveNodes(struct results *rels) {

	int i;
	double avgActiveNodes = 0;
	struct chromosome *chromoTemp;

	for (i = 0; i < getNumChromosomes(rels); i++) {

		chromoTemp = rels->bestChromosomes[i];

		avgActiveNodes += getNumChromosomeActiveNodes(chromoTemp);
	}

	avgActiveNodes = avgActiveNodes / getNumChromosomes(rels);

	return avgActiveNodes;
}


/*
	returns the median number of chromosome active nodes from repeated
	run results specified in rels.
*/
DLL_EXPORT double getMedianActiveNodes(struct results *rels) {

	int i;
	double medActiveNodes = 0;

	int *array = (int*)malloc(getNumChromosomes(rels) * sizeof(int));

	for (i = 0; i < getNumChromosomes(rels); i++) {
		array[i] = getNumChromosomeActiveNodes(rels->bestChromosomes[i]);
	}

	medActiveNodes = medianInt(array, getNumChromosomes(rels));

	free(array);

	return medActiveNodes;
}



/*
	returns the average chromosome fitness from repeated
	run results specified in rels.
*/
DLL_EXPORT double getAverageFitness(struct results *rels) {

	int i;
	double avgFit = 0;
	struct chromosome *chromoTemp;


	for (i = 0; i < getNumChromosomes(rels); i++) {

		chromoTemp = rels->bestChromosomes[i];

		avgFit += getChromosomeFitness(chromoTemp);
	}

	avgFit = avgFit / getNumChromosomes(rels);

	return avgFit;
}


/*
	returns the median chromosome fitness from repeated
	run results specified in rels.
*/
DLL_EXPORT double getMedianFitness(struct results *rels) {

	int i;
	double med = 0;

	double *array = (double*)malloc(getNumChromosomes(rels) * sizeof(double));

	for (i = 0; i < getNumChromosomes(rels); i++) {
		array[i] = getChromosomeFitness(rels->bestChromosomes[i]);
	}

	med = medianDouble(array, getNumChromosomes(rels));

	free(array);

	return med;
}



/*
	returns the average number of generations used by each run  specified in rels.
*/
DLL_EXPORT double getAverageGenerations(struct results *rels) {

	int i;
	double avgGens = 0;
	struct chromosome *chromoTemp;

	for (i = 0; i < getNumChromosomes(rels); i++) {

		chromoTemp = rels->bestChromosomes[i];

		avgGens += getChromosomeGenerations(chromoTemp);
	}

	avgGens = avgGens / getNumChromosomes(rels);

	return avgGens;
}


/*
	returns the median number of generations used by each run  specified in rels.
*/
DLL_EXPORT double getMedianGenerations(struct results *rels) {

	int i;
	double med = 0;

	int *array = (int*)malloc(getNumChromosomes(rels) * sizeof(int));

	for (i = 0; i < getNumChromosomes(rels); i++) {
		array[i] = getChromosomeGenerations(rels->bestChromosomes[i]);
	}

	med = medianInt(array, getNumChromosomes(rels));

	free(array);

	return med;
}



/*
	returns a pointer to a copy of the best chromosomes found on the given run in rels.
*/
DLL_EXPORT struct chromosome* getChromosome(struct results *rels, int run) {

	struct chromosome *chromo;

	/* do some error checking */
	if (rels == NULL) {
		printf("Error: cannot get best chromosome from uninitialised results.\nTerminating CGP-Library.\n");
		exit(0);
	}

	chromo = initialiseChromosomeFromChromosome(rels->bestChromosomes[run]);

	return chromo;
}





/*
	CGP Functions
*/


/*
	Other Functions
*/








/*
	Mutation Methods
*/



/*
	Conductions point mutation on the give chromosome. A predetermined
	number of chromosome genes are randomly selected and changed to
	a random valid allele. The number of mutations is the number of chromosome
	genes multiplied by the mutation rate. Each gene has equal probability
	of being selected.

	DO NOT USE WITH ANN
*/
static void pointMutation(struct parameters *params, struct chromosome *chromo) {

	int i;
	int numGenes;
	int numFunctionGenes, numInputGenes, numOutputGenes, numArgumentGenes;
	int numGenesToMutate;
	int geneToMutate;
	int nodeIndex;
	int nodeInputIndex;
    int nodeArgumentIndex;

	/* get the number of each type of gene */
	numFunctionGenes = chromo->numNodes;
	numInputGenes = chromo->numNodes * params->arity;
	numOutputGenes = params->numOutputs;
    numArgumentGenes = chromo->numNodes * NUM_ARGUMENTS;

	/* set the total number of chromosome genes */
	numGenes = numFunctionGenes + numInputGenes + numOutputGenes + numArgumentGenes;

	/* calculate the number of genes to mutate */
	numGenesToMutate = (int)roundf(numGenes * params->mutationRate);

	/* for the number of genes to mutate */
	for (i = 0; i < numGenesToMutate; i++) {

		/* select a random gene */
		geneToMutate = randInt(numGenes);

		/* mutate function gene */
		if (geneToMutate < numFunctionGenes) {

			nodeIndex = geneToMutate;

			chromo->nodes[nodeIndex]->function = getRandomFunction(chromo->funcSet->numFunctions);
		}

		/* mutate node input gene */
		else if (geneToMutate < numFunctionGenes + numInputGenes) {

			nodeIndex = (int) ((geneToMutate - numFunctionGenes) / chromo->arity);
			nodeInputIndex = (geneToMutate - numFunctionGenes) % chromo->arity;

			chromo->nodes[nodeIndex]->inputs[nodeInputIndex] = getRandomNodeInput(chromo->numInputs, chromo->numNodes, nodeIndex, chromo->recurrentConnectionProbability, chromo->smNodes);
		}

		/* mutate output gene */
		else if (geneToMutate < numFunctionGenes + numInputGenes + numOutputGenes) {
			nodeIndex = geneToMutate - numFunctionGenes - numInputGenes;
			chromo->outputNodes[nodeIndex] = getRandomChromosomeOutput(chromo->numInputs, chromo->numNodes, params->shortcutConnections, chromo->smNodes);
		}
        else{
            nodeIndex = (int)((geneToMutate - numFunctionGenes - numInputGenes - numOutputGenes) / NUM_ARGUMENTS);
            nodeArgumentIndex = (geneToMutate - numFunctionGenes - numInputGenes - numOutputGenes) % NUM_ARGUMENTS;

            chromo->nodes[nodeIndex]->arguments[nodeArgumentIndex] = (int)_randFloat(0,NULL,NULL)*chromo->numNodes;
        }
	}
}


/*
	Same as pointMutation but also mutates weight genes. The reason this is separated is
	that point mutation should always mutate the same number of genes. When weight genes are not
	used many mutations will not do anything and so the number of actual mutations varies.
	- needs explaining better...
*/
static void pointMutationANN(struct parameters *params, struct chromosome *chromo) {

	int i;
	int numGenes;
	int numFunctionGenes, numInputGenes, numWeightGenes, numOutputGenes;
	int numGenesToMutate;
	int geneToMutate;
	int nodeIndex;
	int nodeInputIndex;

	/* get the number of each type of gene */
	numFunctionGenes = params->numNodes;
	numInputGenes = params->numNodes * params->arity;
	numWeightGenes = params->numNodes * params->arity;
	numOutputGenes = params->numOutputs;

	/* set the total number of chromosome genes */
	numGenes = numFunctionGenes + numInputGenes + numWeightGenes + numOutputGenes;

	/* calculate the number of genes to mutate */
	numGenesToMutate = (int)roundf(numGenes * params->mutationRate);

	/* for the number of genes to mutate */
	for (i = 0; i < numGenesToMutate; i++) {

		/* select a random gene */
		geneToMutate = randInt(numGenes);

		/* mutate function gene */
		if (geneToMutate < numFunctionGenes) {

			nodeIndex = geneToMutate;

			chromo->nodes[nodeIndex]->function = getRandomFunction(chromo->funcSet->numFunctions);
		}

		/* mutate node input gene */
		else if (geneToMutate < numFunctionGenes + numInputGenes) {

			nodeIndex = (int) ((geneToMutate - numFunctionGenes) / chromo->arity);
			nodeInputIndex = (geneToMutate - numFunctionGenes) % chromo->arity;

			chromo->nodes[nodeIndex]->inputs[nodeInputIndex] = getRandomNodeInput(chromo->numInputs, chromo->numNodes, nodeIndex, params->recurrentConnectionProbability, chromo->smNodes);
		}

		/* mutate connection weight */
		else if (geneToMutate < numFunctionGenes + numInputGenes + numWeightGenes) {

			nodeIndex = (int) ((geneToMutate - numFunctionGenes - numInputGenes) / chromo->arity);
			nodeInputIndex = (geneToMutate - numFunctionGenes - numInputGenes) % chromo->arity;

			chromo->nodes[nodeIndex]->weights[nodeInputIndex] = getRandomConnectionWeight(params->connectionWeightRange);
		}

		/* mutate output gene */
		else {
			nodeIndex = geneToMutate - numFunctionGenes - numInputGenes - numWeightGenes;
			chromo->outputNodes[nodeIndex] = getRandomChromosomeOutput(chromo->numInputs, chromo->numNodes, params->shortcutConnections, chromo->smNodes);
		}
	}
}



/*
	Conductions a single active mutation on the give chromosome.

	DO NOT USE WITH ANN
*/
static void singleMutation(struct parameters *params, struct chromosome *chromo) {

	int numFunctionGenes, numInputGenes, numOutputGenes;
	int numGenes;
	int geneToMutate;
	int nodeIndex;
	int nodeInputIndex;

	int mutatedActive = 0;
	int previousGeneValue;
	int newGeneValue;

	/* get the number of each type of gene */
	numFunctionGenes = params->numNodes;
	numInputGenes = params->numNodes * params->arity;
	numOutputGenes = params->numOutputs;

	/* set the total number of chromosome genes */
	numGenes = numFunctionGenes + numInputGenes + numOutputGenes;

	/* while active gene not mutated */
	while (mutatedActive == 0) {

		/* select a random gene */
		geneToMutate = randInt(numGenes);

		/* mutate function gene */
		if (geneToMutate < numFunctionGenes) {

			nodeIndex = geneToMutate;

			previousGeneValue = chromo->nodes[nodeIndex]->function;

			chromo->nodes[nodeIndex]->function = getRandomFunction(chromo->funcSet->numFunctions);

			newGeneValue = chromo->nodes[nodeIndex]->function;

			if ((previousGeneValue != newGeneValue) && (chromo->nodes[nodeIndex]->active == 1)) {
				mutatedActive = 1;
			}

		}

		/* mutate node input gene */
		else if (geneToMutate < numFunctionGenes + numInputGenes) {

			nodeIndex = (int) ((geneToMutate - numFunctionGenes) / chromo->arity);
			nodeInputIndex = (geneToMutate - numFunctionGenes) % chromo->arity;

			previousGeneValue = chromo->nodes[nodeIndex]->inputs[nodeInputIndex];

			chromo->nodes[nodeIndex]->inputs[nodeInputIndex] = getRandomNodeInput(chromo->numInputs, chromo->numNodes, nodeIndex, params->recurrentConnectionProbability, chromo->smNodes);

			newGeneValue = chromo->nodes[nodeIndex]->inputs[nodeInputIndex];

			if ((previousGeneValue != newGeneValue) && (chromo->nodes[nodeIndex]->active == 1)) {
				mutatedActive = 1;
			}
		}

		/* mutate output gene */
		else {
			nodeIndex = geneToMutate - numFunctionGenes - numInputGenes;

			previousGeneValue = chromo->outputNodes[nodeIndex];

			chromo->outputNodes[nodeIndex] = getRandomChromosomeOutput(chromo->numInputs, chromo->numNodes, params->shortcutConnections, chromo->smNodes);

			newGeneValue = chromo->outputNodes[nodeIndex];

			if (previousGeneValue != newGeneValue) {
				mutatedActive = 1;
			}
		}
	}
}


/*
	Conductions probabilistic mutation on the given chromosome. Each chromosome
	gene is changed to a random valid allele with a probability specified in
	parameters.
*/
static void probabilisticMutation(struct parameters *params, struct chromosome *chromo) {

	int i, j;

	/* for every nodes in the chromosome */
	for (i = 0; i < chromo->numNodes; i++) {

		/* mutate the function gene */
		if (randDecimal() <= params->mutationRate) {
			chromo->nodes[i]->function = getRandomFunction(chromo->funcSet->numFunctions);
			chromo->smNodes[i] = chromo->funcSet->smFunctions[chromo->nodes[i]->function] == NULL ? 0 : 1;
			//todo: fixNodeInputs(chromo);
		}

		/* for every input to each chromosome */
		for (j = 0; j < params->arity; j++) {

			/* mutate the node input */
			if (randDecimal() <= params->mutationRate) {
				chromo->nodes[i]->inputs[j] = getRandomNodeInput(chromo->numInputs, chromo->numNodes, i, params->recurrentConnectionProbability, chromo->smNodes);
			}

			/* mutate the node connection weight */
			if (randDecimal() <= params->mutationRate) {
				chromo->nodes[i]->weights[j] = getRandomConnectionWeight(params->connectionWeightRange);
			}
		}
	}

	/* for every chromosome output */
	for (i = 0; i < params->numOutputs; i++) {

		/* mutate the chromosome output */
		if (randDecimal() <= params->mutationRate) {
			do {
				chromo->outputNodes[i] = getRandomChromosomeOutput(chromo->numInputs, chromo->numNodes,
																   params->shortcutConnections, chromo->smNodes);
			}while (chromo->outputNodes[i] > chromo->numInputs && chromo->smNodes[chromo->outputNodes[i]-chromo->numInputs] == 1);
		}
	}
}

/*
	Conductions probabilistic mutation on the active nodes in the given
	chromosome. Each chromosome gene is changed to a random valid allele
	with a probability specified in parameters.
*/
static void probabilisticMutationOnlyActive(struct parameters *params, struct chromosome *chromo) {

	int i, j;
	int activeNode;

	/* for every active node in the chromosome */
	for (i = 0; i < chromo->numActiveNodes; i++) {

		activeNode = chromo->activeNodes[i];

		/* mutate the function gene */
		if (randDecimal() <= params->mutationRate) {
			chromo->nodes[activeNode]->function = getRandomFunction(chromo->funcSet->numFunctions);
		}

		/* for every input to each chromosome */
		for (j = 0; j < params->arity; j++) {

			/* mutate the node input */
			if (randDecimal() <= params->mutationRate) {
				chromo->nodes[activeNode]->inputs[j] = getRandomNodeInput(chromo->numInputs, chromo->numNodes, activeNode, params->recurrentConnectionProbability, chromo->smNodes);
			}

			/* mutate the node connection weight */
			if (randDecimal() <= params->mutationRate) {
				chromo->nodes[activeNode]->weights[j] = getRandomConnectionWeight(params->connectionWeightRange);
			}
		}
	}

	/* for every chromosome output */
	for (i = 0; i < params->numOutputs; i++) {

		/* mutate the chromosome output */
		if (randDecimal() <= params->mutationRate) {
			chromo->outputNodes[i] = getRandomChromosomeOutput(chromo->numInputs, chromo->numNodes, params->shortcutConnections, chromo->smNodes);
		}
	}
}



/*
	Sets the random number seed
*/
DLL_EXPORT void setRandomNumberSeed(unsigned int seed) {
	srand(seed);
}


/*
	repetitively applies runCGP to obtain average behaviour
*/
DLL_EXPORT struct results* repeatCGP(struct parameters *params, struct dataSet *data, int numGens, int numRuns) {

	int i;
	struct results *rels;
	int updateFrequency = params->updateFrequency;

	/* set the update frequency so as to to so generational results */
	params->updateFrequency = 0;

	rels = initialiseResults(params, numRuns);

	printf("Run\tFitness\t\tGenerations\tActive Nodes\n");

	/* for each run */
	//#pragma omp parallel for default(none), shared(numRuns,rels,params,data,numGens), schedule(dynamic), num_threads(params->numThreads)
	for (i = 0; i < numRuns; i++) {

		/* run cgp */
		rels->bestChromosomes[i] = runCGP(params, data, numGens);

		printf("%d\t%f\t%d\t\t%d\n", i, rels->bestChromosomes[i]->fitness, rels->bestChromosomes[i]->generation, rels->bestChromosomes[i]->numActiveNodes);
	}

	printf("----------------------------------------------------\n");
	printf("MEAN\t%f\t%f\t%f\n", getAverageFitness(rels), getAverageGenerations(rels), getAverageActiveNodes(rels));
	printf("MEDIAN\t%f\t%f\t%f\n", getMedianFitness(rels), getMedianGenerations(rels), getMedianActiveNodes(rels));
	printf("----------------------------------------------------\n\n");

	/* restore the original value for the update frequency */
	params->updateFrequency = updateFrequency;

	return rels;
}


DLL_EXPORT struct chromosome* runCGP(struct parameters *params, struct dataSet *data, int numGens) {

	int i;
	int gen;

    FILE *fpGenFit = fopen("exp11/FitGen.txt", "a+");

	/* bestChromo found using runCGP */
	struct chromosome *bestChromo;

	/* arrays of the parents and children */
	struct chromosome **parentChromos;
	struct chromosome **childrenChromos;

	/* storage for chromosomes used by selection scheme */
	struct chromosome **candidateChromos;
	int numCandidateChromos;

	/* error checking */
	if (numGens < 0) {
		printf("Error: %d generations is invalid. The number of generations must be >= 0.\n Terminating CGP-Library.\n", numGens);
		exit(0);
	}

	if (data != NULL && params->numInputs != data->numInputs) {
		printf("Error: The number of inputs specified in the dataSet (%d) does not match the number of inputs specified in the parameters (%d).\n", data->numInputs, params->numInputs);
		printf("Terminating CGP-Library.\n");
		exit(0);
	}

	if (data != NULL && params->numOutputs != data->numOutputs) {
		printf("Error: The number of outputs specified in the dataSet (%d) does not match the number of outputs specified in the parameters (%d).\n", data->numOutputs, params->numOutputs);
		printf("Terminating CGP-Library.\n");
		exit(0);
	}

	/* initialise parent chromosomes */
	parentChromos = (struct chromosome**)malloc(params->mu * sizeof(struct chromosome*));

	for (i = 0; i < params->mu; i++) {
		parentChromos[i] = initialiseChromosome(params);
	}

	/* initialise children chromosomes */
	childrenChromos = (struct chromosome**)malloc(params->lambda * sizeof(struct chromosome*));

	for (i = 0; i < params->lambda; i++) {
		childrenChromos[i] = initialiseChromosome(params);
	}

	/* intilise best chromosome */
	bestChromo = initialiseChromosome(params);

	/* determine the size of the Candidate Chromos based on the evolutionary Strategy */
	if (params->evolutionaryStrategy == '+') {
		numCandidateChromos = params->mu + params->lambda;
	}
	else if (params->evolutionaryStrategy == ',') {
		numCandidateChromos = params->lambda;
	}
	else {
		printf("Error: the evolutionary strategy '%c' is not known.\nTerminating CGP-Library.\n", params->evolutionaryStrategy);
		exit(0);
	}

	/* initialise the candidateChromos */
	candidateChromos = (struct chromosome**)malloc(numCandidateChromos * sizeof(struct chromosome*));

	for (i = 0; i < numCandidateChromos; i++) {
		candidateChromos[i] = initialiseChromosome(params);
	}

	/* set fitness of the parents */
	for (i = 0; i < params->mu; i++) {
		setChromosomeFitness(params, parentChromos[i], data);
	}

	/* show the user whats going on */
	if (params->updateFrequency != 0) {
		printf("\n-- Starting CGP --\n\n");
		printf("Gen\tfitness\n");
	}

	/* for each generation */
	for (gen = 0; gen < numGens; gen++) {
		/* set fitness of the children of the population */
		#pragma omp parallel for default(none), shared(params, childrenChromos, data), schedule(static,1), num_threads(params->numThreads)
		for (i = 0; i < params->lambda; i++) {
			setChromosomeFitness(params, childrenChromos[i], data);
		}

		/* get best chromosome */
		getBestChromosome(parentChromos, childrenChromos, params->mu, params->lambda, bestChromo);

		/* check termination conditions */
		if (getChromosomeFitness(bestChromo) <= params->targetFitness) {

			if (params->updateFrequency != 0) {
				printf("%d\t%f - Solution Found\n", gen, bestChromo->fitness);
			}

			break;
		}

		/* display progress to the user at the update frequency specified */
		if (params->updateFrequency != 0 && (gen % params->updateFrequency == 0 || gen >= numGens - 1) && gen < numGens+1 ) {
			printf("%d\t%f\n", gen, bestChromo->fitness);
            fprintf(fpGenFit, "%f,", bestChromo->fitness);
		}

		/*
			Set the chromosomes which will be used by the selection scheme
			dependant upon the evolutionary strategy. i.e. '+' all are used
			by the selection scheme, ',' only the children are.
		*/
		if (params->evolutionaryStrategy == '+') {

			/*
				Note: the children are placed before the parents to
				ensure 'new blood' is always selected over old if the
				fitness are equal.
			*/

			for (i = 0; i < numCandidateChromos; i++) {

				if (i < params->lambda) {
					copyChromosome(candidateChromos[i], childrenChromos[i] );
				}
				else {
					copyChromosome(candidateChromos[i], parentChromos[i - params->lambda] );
				}
			}
		}
		else if (params->evolutionaryStrategy == ',') {

			for (i = 0; i < numCandidateChromos; i++) {
				copyChromosome(candidateChromos[i], childrenChromos[i] );
			}
		}

		/* select the parents from the candidateChromos */
		params->selectionScheme(params, parentChromos, candidateChromos, params->mu, numCandidateChromos);

		/* create the children from the parents */
		params->reproductionScheme(params, parentChromos, childrenChromos, params->mu, params->lambda);

        /* copy the best best chromosome */
        for(i = 0; i<params->lambda;i++){
            childrenChromos[i]->generation = gen+1;
        }
	}

	/* deal with formatting for displaying progress */
	if (params->updateFrequency != 0) {
		printf("\n");
	}

	/* copy the best best chromosome */
	bestChromo->generation = gen;
	/*copyChromosome(chromo, bestChromo);*/

	/* free parent chromosomes */
	for (i = 0; i < params->mu; i++) {
		freeChromosome(parentChromos[i]);
	}
	free(parentChromos);

	/* free children chromosomes */
	for (i = 0; i < params->lambda; i++) {
		freeChromosome(childrenChromos[i]);
	}
	free(childrenChromos);

	/* free the used chromosomes and population */
	for (i = 0; i < numCandidateChromos; i++) {
		freeChromosome(candidateChromos[i]);
	}
	free(candidateChromos);

    fprintf(fpGenFit, "\n");
    fclose(fpGenFit);
	return bestChromo;
}

/*
	returns a pointer to the fittest chromosome in the two arrays of chromosomes

	loops through parents and then the children in order for the children to always be selected over the parents
*/
static void getBestChromosome(struct chromosome **parents, struct chromosome **children, int numParents, int numChildren, struct chromosome *best) {

	int i;
	struct chromosome *bestChromoSoFar;

	bestChromoSoFar = parents[0];

	for (i = 1; i < numParents; i++) {

		if (parents[i]->fitness <= bestChromoSoFar->fitness) {
			bestChromoSoFar = parents[i];
		}
	}

	for (i = 0; i < numChildren; i++) {

		if (children[i]->fitness <= bestChromoSoFar->fitness) {
			bestChromoSoFar = children[i];
		}
	}

	copyChromosome(best, bestChromoSoFar);
}


/*
	copies the contents of funcSetSrc to funcSetDest
*/
static void copyFunctionSet(struct functionSet *funcSetDest, struct functionSet *funcSetSrc) {

	int i;

	funcSetDest->numFunctions = funcSetSrc->numFunctions;

	for (i = 0; i < funcSetDest->numFunctions; i++) {
		strncpy(funcSetDest->functionNames[i], funcSetSrc->functionNames[i], FUNCTIONNAMELENGTH);
		funcSetDest->functions[i] = funcSetSrc->functions[i];
        funcSetDest->infoFunctions[i] = funcSetSrc-> infoFunctions[i];
        funcSetDest->smFunctions[i] = funcSetSrc->smFunctions[i];
		funcSetDest->maxNumInputs[i] = funcSetSrc->maxNumInputs[i];
        funcSetDest->type[i] = funcSetSrc->type[i];
	}
}


/*
	copys the contents from the src node into dest node.
*/
void copyNode(struct node *nodeDest, struct node *nodeSrc) {

	int i;

	/* copy the node's function */
	nodeDest->function = nodeSrc->function;

	/* copy active flag */
	nodeDest->active = nodeSrc->active;

	/* copy the node arity */
	nodeDest->maxArity = nodeSrc->maxArity;
	nodeDest->actArity = nodeSrc->actArity;

	/* copy the nodes inputs and connection weights */
	for (i = 0; i < nodeSrc->maxArity; i++) {
		nodeDest->inputs[i] = nodeSrc->inputs[i];
		nodeDest->weights[i] = nodeSrc->weights[i];
	}

	for (i = 0; i < NUM_ARGUMENTS; i++){
		nodeDest->arguments[i] = nodeSrc->arguments[i];
	}

    nodeDest->output = nodeSrc->output;

}


/*
	mutate Random parent reproduction method.
*/
static void mutateRandomParent(struct parameters *params, struct chromosome **parents, struct chromosome **children, int numParents, int numChildren) {

	int i;

	/* for each child */
	for (i = 0; i < numChildren; i++) {

		/* set child as clone of random parent */
		copyChromosome(children[i], parents[randInt(numParents)]);

		/* mutate newly cloned child */
		mutateChromosome(params, children[i]);
	}
}


/*
	Selection scheme which selects the fittest members of the population
	to be the parents.

	The candidateChromos contains the current children followed by the
 	current parents. This means that using a stable sort to order
	candidateChromos results in children being selected over parents if
	their fitnesses are equal. A desirable property in CGP to facilitate
	neutral genetic drift.
*/
static void selectFittest(struct parameters *params, struct chromosome **parents, struct chromosome **candidateChromos, int numParents, int numCandidateChromos ) {

	int i;

	sortChromosomeArray(candidateChromos, numCandidateChromos);

	for (i = 0; i < numParents; i++) {
		copyChromosome(parents[i], candidateChromos[i]);
	}
}



/*
	returns a pointer to an initialised node. Initialised means that necessary
	memory has been allocated and values set.
*/
struct node *initialiseNode(int numInputs, int numNodes, int arity, int numFunctions, double connectionWeightRange, double recurrentConnectionProbability, int nodePosition, int *smNodes) {

	struct node *n;
	int i;

	/* allocate memory for node */
	n = (struct node*)malloc(sizeof(struct node));

	/* allocate memory for the node's inputs and connection weights */
	n->inputs = (int*)malloc(arity * sizeof(int));
	n->weights = (double*)malloc(arity * sizeof(double));

	/* set the node's function */
	n->function = getRandomFunction(numFunctions);

    for(i = 0; i < NUM_ARGUMENTS; i++){
        float f = _randFloat(0,NULL,NULL)*numNodes;
        n->arguments[i] = (int)f;
        //printf("rand float = %f converted to %d\n",f,n->arguments[i]);
    }

    /* set as active by default */
	n->active = 1;

	/* set the nodes inputs and connection weights */
	for (i = 0; i < arity; i++) {
		int in = getRandomNodeInput(numInputs, numNodes, nodePosition, recurrentConnectionProbability, smNodes);
        //printf("%d->%d\n", nodePosition, in);
		n->inputs[i] = in;
		n->weights[i] = getRandomConnectionWeight(connectionWeightRange);
	}

	/* set the output of the node to zero*/
	n->output = 0.0;

	/* set the arity of the node */
	n->maxArity = arity;

	return n;
}


/*
	Free memory associated with given node
*/
void freeNode(struct node *n) {

	/* attempt to prevent user double freeing */
	if (n == NULL) {
		printf("Warning: double freeing of node prevented.\n");
		return;
	}

	free(n->inputs);
	free(n->weights);
	free(n);
	//n = NULL;
}

/*
	returns a random connection weight value
*/
static double getRandomConnectionWeight(double weightRange) {
	return weightRange;//(randDecimal() * 2 * weightRange) - weightRange;
}

/*
	returns a random function index
*/
static int getRandomFunction(int numFunctions) {

	/* check that funcSet contains functions */
	if (numFunctions < 1) {
		printf("Error: cannot assign the function gene a value as the Function Set is empty.\nTerminating CGP-Library.\n");
		exit(0);
	}

	return randInt(numFunctions);
}

/*
	returns a random input for the given node
*/
static int getRandomNodeInput(int numChromoInputs, int numNodes, int nodePosition, double recurrentConnectionProbability, int *smNodes) {

	int input;
    double rand = randDecimal();
    int count = 0;
    do {
//        if (rand < recurrentConnectionProbability && count < 5)
//            input = randInt(8)-4+nodePosition;
//        else
//            input = randInt(8)-8+nodePosition;
        /* pick any ahead nodes or the node itself */
        if (rand < recurrentConnectionProbability) {
            //printf("ra %f, rec%f",rand, recurrentConnectionProbability);
            input = randInt(numNodes - nodePosition) + nodePosition + 1;
        }
            /* pick any previous node including inputs */
        else {
            input = randInt(numChromoInputs + nodePosition);
            //printf("ra %f",rand);
        }
        //printf("   %d  sm %d\n",input, smNodes[input-numChromoInputs]);
         count++;
        if(count > 5) return 0;
//        if(input >= numNodes) input = numNodes-1;
//        if(input < 0) input = 0;
    }while( input >= numChromoInputs && smNodes[input-numChromoInputs] == 1);
    //printf("n:%d,in:%d\n",nodePosition,input);
    return input;
}


/*
	returns a random chromosome output
*/
static int getRandomChromosomeOutput(int numInputs, int numNodes, int shortcutConnections, int *smNodes) {

	int output;

    do{

	if ( shortcutConnections == 1)
		output = randInt(numInputs + numNodes);
	else
		output = randInt(numNodes) + numInputs;
    }while (output >= numInputs && smNodes[output-numInputs] == 1);

	return output;
}



/*
	The default fitness function used by CGP-Library.
	Simply assigns an error of the sum of the absolute differences between the target and actual outputs for all outputs over all samples
*/
static double supervisedLearning(struct parameters *params, struct chromosome *chromo, struct dataSet *data) {

	int i, j;
	double error = 0;

	/* error checking */
	if (getNumChromosomeInputs(chromo) != getNumDataSetInputs(data)) {
		printf("Error: the number of chromosome inputs must match the number of inputs specified in the dataSet.\n");
		printf("Terminating CGP-Library.\n");
		exit(0);
	}

	if (getNumChromosomeOutputs(chromo) != getNumDataSetOutputs(data)) {
		printf("Error: the number of chromosome outputs must match the number of outputs specified in the dataSet.\n");
		printf("Terminating CGP-Library.\n");
		exit(0);
	}

	/* for each sample in data */
	for (i = 0 ; i < getNumDataSetSamples(data); i++) {

		/* calculate the chromosome outputs for the set of inputs  */
		executeChromosome(chromo, getDataSetSampleInputs(data, i));

		/* for each chromosome output */
		for (j = 0; j < getNumChromosomeOutputs(chromo); j++) {

			error += fabs(getChromosomeOutput(chromo, j) - getDataSetSampleOutput(data, i, j));
		}
	}

	return error;
}

/*
	Prints the current functions in the function set to
	the terminal.
*/
static void printFunctionSet(struct parameters *params) {

    int i;

    printf("Function Set:");

    for (i = 0; i < params->funcSet->numFunctions; i++) {
        printf(" %s", params->funcSet->functionNames[i]);
    }

    printf(" (%d)\n", params->funcSet->numFunctions);
}

void checkChromosomeInputs(struct chromosome* chromo){

	int i,j;
	int num_nodes = chromo->numNodes;
	float recurrent = chromo->recurrentConnectionProbability;

	// Check inputs to nodes
	for(i = 0; i < num_nodes; i++){

		chromo->nodes[i]->actArity = getChromosomeNodeArity(chromo, i);

		for(j = 0; j < chromo->nodes[i]->actArity; j++) {
			//int oldInput = chromo->nodes[i]->inputs[j];

			if(recurrent == 0.0){
				if(chromo->nodes[i]->inputs[j] >= i + chromo->numInputs ){
					chromo->nodes[i]->inputs[j] = i + chromo->numInputs - 2;
				}

			}else{
				if (chromo->nodes[i]->inputs[j] >= num_nodes + chromo->numInputs) {
					chromo->nodes[i]->inputs[j] = num_nodes + chromo->numInputs - 1;
					// if input is self, use previous node
				}else if(chromo->nodes[i]->inputs[j] < 0){
                    chromo->nodes[i]->inputs[j] = 0;
                }
			}

            //check sm-node

            //if(chromo->smNodes[chromo->nodes[i]->inputs[j]-chromo->numInputs] == 1){
              //  chromo->nodes[i]->inputs[j] = getRandomNodeInput(chromo->numInputs, chromo->numNodes, i, recurrent, chromo->smNodes);
                //printf("\x1b[33m FIX: Node %d moved input from %d to %d\x1b[0m\n", i, oldInput,chromo->nodes[i]->inputs[j]);
            //}

//			if (chromo->nodes[i]->inputs[j] != oldInput)
//				printf("\x1b[33m FIX: Node %d moved input from %d to %d\x1b[0m\n", i, oldInput,
//					   chromo->nodes[i]->inputs[j]);
		}
	}
}